@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998-2000  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2001  Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,57 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: CDD2600.cc,v $
- * Revision 1.2  2000/04/23 16:29:49  andreasm
- * Updated to state of my private development environment.
- *
- * Revision 1.16  1999/12/15 20:31:46  mueller
- * Added remote messages for 'read-cd' progress used by a GUI.
- *
- * Revision 1.15  1999/11/07 09:14:59  mueller
- * Release 1.1.3
- *
- * Revision 1.14  1999/04/05 11:04:10  mueller
- * Added driver option flags.
- *
- * Revision 1.13  1999/03/27 20:52:02  mueller
- * Adapted to changed writing interface.
- *
- * Revision 1.12  1998/10/24 14:30:40  mueller
- * Changed prototype of 'readDiskToc()'.
- *
- * Revision 1.11  1998/10/03 15:11:05  mueller
- * Moved basic writing methods to class 'CDD2600Base'.
- *
- * Revision 1.10  1998/09/27 19:17:46  mueller
- * Fixed 'disc-info' for CDD2000.
- * Added retrieval of control nibbles for track with 'analyzeTrack()'.
- *
- * Revision 1.9  1998/09/22 19:15:13  mueller
- * Removed memory allocations during write process.
- *
- * Revision 1.8  1998/09/08 11:54:22  mueller
- * Extended disk info structure because CDD2000 does not support the
- * 'READ DISK INFO' command.
- *
- * Revision 1.7  1998/09/07 15:20:20  mueller
- * Reorganized read-toc related code.
- *
- * Revision 1.6  1998/09/06 13:34:22  mueller
- * Use 'message()' for printing messages.
- *
- * Revision 1.5  1998/08/25 19:23:46  mueller
- * Added index extraction for 'read-toc'.
- * Added disk info code.
- *
- * Revision 1.4  1998/08/07 12:44:30  mueller
- * Changed group 1 mode select commands to group 0 mode select commands.
- * Implement pre-gap detection in 'readDiscToc()'.
- *
- */
-
-static char rcsid[] = "$Id: CDD2600.cc,v 1.2 2000/04/23 16:29:49 andreasm Exp $";
 
 #include <config.h>
 
@@ -226,7 +175,7 @@ int CDD2600::startDao()
     return 1;
   }
 
-  message(1, "Writing lead-in and gap...");
+  message(2, "Writing lead-in and gap...");
 
   lba = diskInfo_.thisSessionLba - 150 - leadInLength_;
 
@@ -236,7 +185,7 @@ int CDD2600::startDao()
     return 1;
   }
 
-  message(0, "Lba after lead-in: %ld", lba);
+  message(5, "Lba after lead-in: %ld", lba);
 
   // write gap (2 seconds)
   if (writeZeros(toc_->leadInMode(), lba, lba + 150, 150) != 0) {
@@ -244,7 +193,7 @@ int CDD2600::startDao()
     return 1;
   }
 
-  message(1, "");
+  message(2, "");
 
   return 0;
 }
@@ -253,7 +202,7 @@ int CDD2600::finishDao()
 {
   long lba = diskInfo_.thisSessionLba + toc_->length().lba();
 
-  message(1, "Writing lead-out...");
+  message(2, "Writing lead-out...");
 
   // write lead-out
   if (writeZeros(toc_->leadOutMode(), lba, lba + 150, leadOutLength_) != 0) {
@@ -261,13 +210,13 @@ int CDD2600::finishDao()
     return 1;
   }
 
-  message(1, "\nFlushing cache...");
+  message(2, "\nFlushing cache...");
   
   if (flushCache() != 0) {
     return 1;
   }
 
-  message(1, "");
+  message(2, "");
 
   blockLength_ = MODE1_BLOCK_LEN;
   modeSelectBlockSize(blockLength_, 1);
@@ -543,7 +492,6 @@ int CDD2600::nextWritableAddress(long *lba, int showError)
 DiskInfo *CDD2600::diskInfo()
 {
   unsigned char cmd[10];
-  unsigned long dataLen = 34;
   unsigned char data[34];
   long nwa;
   int i;
@@ -577,11 +525,11 @@ DiskInfo *CDD2600::diskInfo()
   cmd[8] = 4;
 
   if (sendCmd(cmd, 10, NULL, 0, data, 4, 0) == 0) {
-    message(3, "First track %u, last track %u", data[2], data[3]);
+    message(5, "First track %u, last track %u", data[2], data[3]);
     diskInfo_.lastTrackNr = data[3];
   }
   else {
-    message(3, "READ TOC (format 0) failed.");
+    message(5, "READ TOC (format 0) failed.");
   }
 
   if (diskInfo_.lastTrackNr > 0) {
@@ -592,7 +540,7 @@ DiskInfo *CDD2600::diskInfo()
     diskInfo_.diskTocType = 0xff; // undefined
 
     if (diskInfo_.lastTrackNr < 99 && nextWritableAddress(&nwa, 0) == 0) {
-      message(0, "NWA: %ld", nwa);
+      message(5, "NWA: %ld", nwa);
       diskInfo_.thisSessionLba = nwa;
       diskInfo_.append = 1;
     }
@@ -614,11 +562,11 @@ DiskInfo *CDD2600::diskInfo()
       diskInfo_.lastSessionLba = (data[8] << 24) | (data[9] << 16) |
                                  (data[10] << 8) | data[11];
 
-      message(3, "First session %u, last session %u, last session start %ld",
+      message(5, "First session %u, last session %u, last session start %ld",
 	      data[2], data[3], diskInfo_.lastSessionLba);
     }
     else {
-      message(3, "READ TOC (format 1) failed.");
+      message(5, "READ TOC (format 1) failed.");
     }
 
     if (diskInfo_.sessionCnt > 0) {
@@ -662,7 +610,7 @@ DiskInfo *CDD2600::diskInfo()
 	    diskInfo_.append = 1;
 	}
 	else {
-	  message(3, "Did not find BO pointer in session %d.",
+	  message(4, "Did not find BO pointer in session %d.",
 		  diskInfo_.sessionCnt);
 	  
 	}
@@ -671,7 +619,7 @@ DiskInfo *CDD2600::diskInfo()
 	delete[] toc;
       }
       else {
-	message(3, "getRawToc failed.");
+	message(5, "getRawToc failed.");
       }
     }
   }
@@ -713,7 +661,7 @@ CdRawToc *CDD2600::getRawToc(int sessionNr, int *len)
 
   dataLen = ((reqData[0] << 8) | reqData[1]) + 2;
 
-  message(3, "Raw toc data len: %d", dataLen);
+  message(5, "Raw toc data len: %d", dataLen);
 
   data = new (unsigned char)[dataLen];
   
@@ -733,7 +681,7 @@ CdRawToc *CDD2600::getRawToc(int sessionNr, int *len)
   rawToc = new CdRawToc[entries];
 
   for (i = 0, p = data + 4; i < entries; i++, p += 11 ) {
-#if 1
+#if 0
     message(0, "%d %02x %02d %2x %02d:%02d:%02d %02x %02d:%02d:%02d",
 	    p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7], p[8], p[9], p[10]);
 #endif
@@ -926,7 +874,7 @@ int CDD2600::readSubChannels(long lba, long len, SubChannel ***chans,
   return 0;
 }
 
-int CDD2600::readAudioRange(int fd, long start, long end,
+int CDD2600::readAudioRange(ReadDiskInfo *rinfo, int fd, long start, long end,
 			    int startTrack, int endTrack,
 			    TrackInfo *info)
 {
@@ -936,22 +884,31 @@ int CDD2600::readAudioRange(int fd, long start, long end,
     message(1, "Analyzing...");
 
     for (t = startTrack; t <= endTrack; t++) {
+      long totalProgress;
 
-      sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 0);
+      totalProgress = t * 1000;
+      totalProgress /= rinfo->tracks;
 
-      message(1, "Track %d...", t + 1);
+      sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 0,
+			    totalProgress);
+
+      message(2, "Track %d...", t + 1);
       info[t].isrcCode[0] = 0;
       readIsrc(t + 1, info[t].isrcCode);
 
       if (info[t].isrcCode[0] != 0)
-	message(1, "Found ISRC code.");
+	message(2, "Found ISRC code.");
 
-      sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 1000);
+      totalProgress = (t + 1) * 1000;
+      totalProgress /= rinfo->tracks;
+
+      sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 1000,
+			    totalProgress);
     }
 
     message(1, "Reading...");
   }
 
-  return CdrDriver::readAudioRangeParanoia(fd, start, end, startTrack,
+  return CdrDriver::readAudioRangeParanoia(rinfo, fd, start, end, startTrack,
 					   endTrack, info);
 }
