@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998-2000  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2001  Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,91 +17,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/*
- * $Log: GenericMMC.cc,v $
- * Revision 1.7  2000/06/22 12:19:28  andreasm
- * Added switch for reading CDs written in TAO mode.
- * The fifo buffer size is now also saved to $HOME/.cdrdao.
- *
- * Revision 1.6  2000/06/19 20:17:37  andreasm
- * Added CDDB reading to add CD-TEXT information to toc-files.
- * Fixed bug in reading ATIP data in 'GenericMMC::diskInfo()'.
- * Attention: CdrDriver.cc is currently configured to read TAO disks.
- *
- * Revision 1.5  2000/06/10 14:48:05  andreasm
- * Tracks that are shorter than 4 seconds can be recorded now if the user confirms
- * it.
- * The driver table is now read from an external file (.../share/cdrdao/drivers
- * and $HOME/.cdrdao-drivers).
- * Fixed bug the might prevented writing pure data CDs with some recorders.
- *
- * Revision 1.4  2000/06/06 22:26:13  andreasm
- * Updated list of supported drives.
- * Added saving of some command line settings to $HOME/.cdrdao.
- * Added test for multi session support in raw writing mode to GenericMMC.cc.
- * Updated manual page.
- *
- * Revision 1.3  2000/05/01 18:13:18  andreasm
- * Fixed too small mode page buffer.
- *
- * Revision 1.2  2000/04/23 16:29:50  andreasm
- * Updated to state of my private development environment.
- *
- * Revision 1.14  1999/12/15 20:31:46  mueller
- * Added remote messages for 'read-cd' progress used by a GUI.
- *
- * Revision 1.13  1999/11/07 09:14:59  mueller
- * Release 1.1.3
- *
- * Revision 1.12  1999/04/05 18:47:40  mueller
- * Added driver options.
- * Added option to read Q sub-channel data instead raw PW sub-channel data
- * for 'read-toc'.
- *
- * Revision 1.11  1999/03/27 20:51:26  mueller
- * Added data track support.
- *
- * Revision 1.10  1998/10/25 14:38:28  mueller
- * Fixed comment.
- *
- * Revision 1.9  1998/10/24 14:48:55  mueller
- * Added retrieval of next writable address. The Panasonic CW-7502 needs it.
- *
- * Revision 1.8  1998/10/03 15:08:44  mueller
- * Moved 'writeZeros()' to base class 'CdrDriver'.
- * Takes 'writeData()' from base class now.
- *
- * Revision 1.7  1998/09/27 19:18:48  mueller
- * Added retrieval of control nibbles for track with 'analyzeTrack()'.
- * Added multi session mode.
- *
- * Revision 1.6  1998/09/22 19:15:13  mueller
- * Removed memory allocations during write process.
- *
- * Revision 1.5  1998/09/08 11:54:22  mueller
- * Extended disk info structure because CDD2000 does not support the
- * 'READ DISK INFO' command.
- *
- * Revision 1.4  1998/09/07 15:20:20  mueller
- * Reorganized read-toc related code.
- *
- * Revision 1.3  1998/09/06 13:34:22  mueller
- * Use 'message()' for printing messages.
- *
- * Revision 1.2  1998/08/30 19:17:56  mueller
- * Fixed cue sheet generation and first writable address after testing
- * with a Yamaha CDR400t.
- *
- * Revision 1.1  1998/08/15 20:44:58  mueller
- * Initial revision
- *
- */
-
-static char rcsid[] = "$Id: GenericMMC.cc,v 1.7 2000/06/22 12:19:28 andreasm Exp $";
-
 #include <config.h>
 
 #include <string.h>
+#include <time.h>
 #include <assert.h>
 
 #include "GenericMMC.h"
@@ -117,7 +36,7 @@ GenericMMC::GenericMMC(ScsiIf *scsiIf, unsigned long options)
   : CdrDriver(scsiIf, options)
 {
   int i;
-  driverName_ = "Generic SCSI-3/MMC - Version 1.0 (data)";
+  driverName_ = "Generic SCSI-3/MMC - Version 1.2";
   
   speed_ = 0;
   simulate_ = 1;
@@ -226,20 +145,48 @@ int GenericMMC::loadUnload(int unload) const
 // Performs complete blanking of a CD-RW.
 // return: 0: OK
 //         1: scsi command failed
-int GenericMMC::blankDisk()
+int GenericMMC::blankDisk(BlankingMode mode)
 {
   unsigned char cmd[12];
+  int ret;
+  time_t startTime, endTime;
 
   memset(cmd, 0, 12);
 
   cmd[0] = 0xa1; // BLANK
-  cmd[1] = 1 << 4; // erase complete disk, immediate return
+
+  switch (mode) {
+  case BLANK_FULL:
+    cmd[1] = 0x0; // erase complete disk
+    break;
+  case BLANK_MINIMAL:
+    cmd[1] = 0x1; // erase PMA, lead-in and 1st track's pre-gap
+    break;
+  }
+
+  cmd[1] |= 1 << 4; // immediate return
 
   if (sendCmd(cmd, 12, NULL, 0, NULL, 0, 1) != 0) {
     message(-2, "Cannot erase CD-RW.");
     return 1;
   }
 
+  time(&startTime);
+
+  do {
+    mSleep(2000);
+
+    ret = testUnitReady(0);
+
+    if (ret == 1) {
+      message(-2, "Test Unit Ready command failed.");
+    }
+  } while (ret == 2);
+
+  time(&endTime);
+
+  message(2, "Blanking time: %ld seconds", endTime - startTime);
+
   return 0;
 }
 
@@ -318,9 +265,9 @@ int GenericMMC::getSessionInfo()
   }
 
 
-  message(3, "Lead-in start: %s length: %ld", leadInStart_.str(),
+  message(4, "Lead-in start: %s length: %ld", leadInStart_.str(),
 	  leadInLen_);
-  message(3, "Lead-out length: %ld", leadOutLen_);
+  message(4, "Lead-out length: %ld", leadOutLen_);
 
   return 0;
 }
@@ -349,20 +296,39 @@ int GenericMMC::readBufferCapacity(long *capacity)
 int GenericMMC::performPowerCalibration()
 {
   unsigned char cmd[10];
+  int ret;
 
   memset(cmd, 0, 10);
 
   cmd[0] = 0x54; // SEND OPC INFORMATION
   cmd[1] = 1;
 
-  message(1, "Executing power calibration...");
+  message(2, "Executing power calibration...");
+
+  if ((ret = sendCmd(cmd, 10, NULL, 0, NULL, 0)) != 0) {
+    if (ret == 2) {
+      const unsigned char *sense;
+      int senseLen;
+
+      sense = scsiIf_->getSense(senseLen);
+
+      if(senseLen >= 14 && (sense[2] & 0x0f) == 0x5 && sense[7] >= 6 &&
+	   sense[12] == 0x20 && sense[13] == 0x0) {
+	message(2, "Power calibration not supported.");
+	return 0;
+      }
+      else {
+	message(-2, "Power calibration failed.");
+      }
+    }
+    else {
+      message(-2, "Power calibration failed.");
+    }
 
-  if (sendCmd(cmd, 10, NULL, 0, NULL, 0) != 0) {
-    message(-2, "Power calibration failed.");
     return 1;
   }
   
-  message(1, "Power calibration successful.");
+  message(2, "Power calibration successful.");
 
   return 0;
 }
@@ -390,6 +356,19 @@ int GenericMMC::setWriteParameters()
     mp[2] |= 1 << 4; // test write
   }
 
+  DriveInfo di;
+  if (driveInfo(&di, 1) == 0 && di.burnProof) {
+    // This drive has BURN-Proof function.
+    // Enable it unless explicitly disabled.
+    if (options_ & OPT_MMC_NO_BURNPROOF) {
+      message(2, "Turning BURN-Proof off");
+      mp[2] &= ~0x40;
+    } else {
+      message(2, "Turning BURN-Proof on");
+      mp[2] |= 0x40;
+    }
+  }
+
   mp[3] &= 0x3f; // Multi-session: No B0 pointer, next session not allowed
 
   if (multiSession_ != 0)
@@ -398,11 +377,20 @@ int GenericMMC::setWriteParameters()
     mp[3] |= 0x01 << 6; // use B0=FF:FF:FF when closing last session of a
                         // multi session CD-R
 
-  message(3, "Multi session mode: %d", mp[3] >> 6);
+  message(4, "Multi session mode: %d", mp[3] >> 6);
 
   mp[4] &= 0xf0; // Data Block Type: raw data, block size: 2352 (I think not
                  // used for session at once writing)
 
+  if (cdTextEncoder_ != NULL) {
+    mp[4] |= 3; /* Data Block Type: raw data with raw P-W sub-channel data,
+		   block size 2448, required for CD-TEXT lead-in writing
+		   according to the SCSI/MMC-3 manual
+		*/
+  }
+
+  message(4, "Data block type: %u",  mp[4] & 0x0f);
+
   mp[8] = sessionFormat();
 
   if (!diskInfo_.empty) {
@@ -416,7 +404,7 @@ int GenericMMC::setWriteParameters()
     }
   }
 
-  message(3, "Toc type: 0x%x", mp[8]);
+  message(4, "Toc type: 0x%x", mp[8]);
 
   if (setModePage(mp, mpHeader, NULL, 1) != 0) {
     message(-2, "Cannot set write parameters mode page.");
@@ -450,20 +438,20 @@ int GenericMMC::getNWA(long *nwa)
   }
 
 #if 0
-  message(2,"Track Information Block");
-  for (int i=0;i<infoblocklen;i++) message(2,"byte %02x : %02x",i,info[i]);
+  message(3,"Track Information Block");
+  for (int i=0;i<infoblocklen;i++) message(3,"byte %02x : %02x",i,info[i]);
 #endif
 
   if ((info[6] & 0x40) && (info[7] & 0x01) && !(info[6] & 0xb0))
   {
-      message(3,"Track is Blank, Next Writable Address is valid");
+      message(4,"Track is Blank, Next Writable Address is valid");
       lba |= info[12] << 24; // MSB of LBA
       lba |= info[13] << 16;
       lba |= info[14] << 8;
       lba |= info[15];       // LSB of LBA
   }
 
-  message(3, "NWA: %ld", lba);
+  message(4, "NWA: %ld", lba);
 
   if (nwa != NULL) 
     *nwa = lba;
@@ -617,9 +605,18 @@ unsigned char *GenericMMC::createCueSheet(long *cueSheetLen)
   }
 
   cueSheet[n*8+4] = 0;    // Serial Copy Management System
-  cueSheet[n*8+5] = 0;    // MIN
-  cueSheet[n*8+6] = 0;    // SEC
-  cueSheet[n*8+7] = 0;    // FRAME
+
+  if (cdTextEncoder_ != NULL) {
+    cueSheet[n*8+5] = leadInStart_.min();
+    cueSheet[n*8+6] = leadInStart_.sec();
+    cueSheet[n*8+7] = leadInStart_.frac();
+  }
+  else {
+    cueSheet[n*8+5] = 0;    // MIN
+    cueSheet[n*8+6] = 0;    // SEC
+    cueSheet[n*8+7] = 0;    // FRAME
+  }
+
   n++;
 
   int firstTrack = 1;
@@ -769,11 +766,11 @@ unsigned char *GenericMMC::createCueSheet(long *cueSheetLen)
   cueSheet[n*8+6] = lostart.sec();
   cueSheet[n*8+7] = lostart.frac();
 
-  message(2, "\nCue Sheet:");
-  message(2, "CTL/  TNO  INDEX  DATA  SCMS  MIN  SEC  FRAME");
-  message(2, "ADR               FORM");
+  message(3, "\nCue Sheet:");
+  message(3, "CTL/  TNO  INDEX  DATA  SCMS  MIN  SEC  FRAME");
+  message(3, "ADR               FORM");
   for (n = 0; n < len; n++) {
-    message(2, "%02x    %02x    %02x     %02x    %02x   %02d   %02d   %02d",
+    message(3, "%02x    %02x    %02x     %02x    %02x   %02d   %02d   %02d",
 	   cueSheet[n*8],
 	   cueSheet[n*8+1], cueSheet[n*8+2], cueSheet[n*8+3], cueSheet[n*8+4],
 	   cueSheet[n*8+5], cueSheet[n*8+6], cueSheet[n*8+7]);
@@ -892,7 +889,7 @@ int GenericMMC::startDao()
   if (sendCueSheet() != 0)
     return 1;
 
-  //message(1, "Writing lead-in and gap...");
+  //message(2, "Writing lead-in and gap...");
 
   if (writeCdTextLeadIn() != 0) {
     return 1;
@@ -911,7 +908,7 @@ int GenericMMC::finishDao()
 {
   int ret;
 
-  //message(1, "Writing lead-out...");
+  //message(2, "Writing lead-out...");
 
   while ((ret = testUnitReady(0)) == 2)
     mSleep(2000);
@@ -919,7 +916,7 @@ int GenericMMC::finishDao()
   if (ret != 0)
     message(-1, "TEST UNIT READY failed after recording.");
   
-  message(1, "Flushing cache...");
+  message(2, "Flushing cache...");
   
   if (flushCache() != 0) {
     return 1;
@@ -1062,26 +1059,30 @@ int GenericMMC::writeCdTextLeadIn()
   unsigned char cmd[10];
   const PWSubChannel96 **cdTextSubChannels;
   long cdTextSubChannelCount;
-  long channelsPerCmd = scsiIf_->maxDataLen() / 96;
+  long channelsPerCmd;
   long scp = 0;
   long lba = -150 - leadInLen_;
   long len = leadInLen_;
   long n;
   long i;
+  int retry;
+  int ret;
   unsigned char *p;
 
   if (cdTextEncoder_ == NULL)
     return 0;
 
+  channelsPerCmd = scsiIf_->maxDataLen() / 96;
+
   cdTextSubChannels = cdTextEncoder_->getSubChannels(&cdTextSubChannelCount);
 
   assert(channelsPerCmd > 0);
   assert(cdTextSubChannels != NULL);
   assert(cdTextSubChannelCount > 0);
 
-  message(1, "Writing CD-TEXT lead-in...");
+  message(2, "Writing CD-TEXT lead-in...");
 
-  message(2, "Start LBA: %ld, length: %ld", lba, len);
+  message(4, "Start LBA: %ld, length: %ld", lba, len);
 
   memset(cmd, 0, 10);
   cmd[0] = 0x2a; // WRITE1
@@ -1108,18 +1109,43 @@ int GenericMMC::writeCdTextLeadIn()
 	scp = 0;
     }
 
-    message(4, "Writing %ld CD-TEXT sub-channels at LBA %ld.", n, lba);
+    message(5, "Writing %ld CD-TEXT sub-channels at LBA %ld.", n, lba);
 
-    if (sendCmd(cmd, 10, transferBuffer_, n * 96, NULL, 0) != 0) {
+    do {
+      retry = 0;
+
+      ret = sendCmd(cmd, 10, transferBuffer_, n * 96, NULL, 0, 0);
+
+      if(ret == 2) {
+        const unsigned char *sense;
+        int senseLen;
+	
+        sense = scsiIf_->getSense(senseLen);
+
+	// check if drive rejected the command because the internal buffer
+	// is filled
+	if(senseLen >= 14 && (sense[2] & 0x0f) == 0x2 && sense[7] >= 6 &&
+	   sense[12] == 0x4 && sense[13] == 0x8) {
+	  // Not Ready, long write in progress
+	  mSleep(40);
+          retry = 1;
+        }
+	else {
+	  scsiIf_->printError();
+	}
+      }
+    } while (retry);
+
+    if (ret != 0) {
       message(-2, "Writing of CD-TEXT data failed.");
       return 1;
     }
-
+      
 
     len -= n;
     lba += n;
   }
-    
+
   return 0;
 }
 
@@ -1130,10 +1156,6 @@ DiskInfo *GenericMMC::diskInfo()
   unsigned char data[34];
   char spd;
 
-  DriveInfo info;
-
-  driveInfo(&info, 1);
-
   memset(&diskInfo_, 0, sizeof(DiskInfo));
 
   // perform READ DISK INFORMATION
@@ -1528,7 +1550,7 @@ int GenericMMC::getFeature(unsigned int feature, unsigned char *buf,
 
   len = (header[0] << 24) | (header[1] << 16) | (header[2] << 8) | header[3];
 
-  message(3, "getFeature: data len: %lu", len);
+  message(4, "getFeature: data len: %lu", len);
 
   if (len < 8)
     return 1; // feature not defined
@@ -1556,7 +1578,7 @@ int GenericMMC::getFeature(unsigned int feature, unsigned char *buf,
   
   len = (header[0] << 24) | (header[1] << 16) | (header[2] << 8) | header[3];
   
-  message(3, "getFeature: data len: %lu", len);
+  message(4, "getFeature: data len: %lu", len);
 
   if (len < 8) {
     delete[] data;
@@ -1581,11 +1603,12 @@ int GenericMMC::driveInfo(DriveInfo *info, int showErrorMsg)
 
   if (getModePage(0x2a, mp, 32, NULL, NULL, showErrorMsg) != 0) {
     if (showErrorMsg) {
-      message(-2, "Cannot retrieve CD capabilities mode page.");
+      message(-2, "Cannot retrieve drive capabilities mode page.");
     }
     return 1;
   }
 
+  info->burnProof = (mp[4] & 0x80) ? 1 : 0;
   info->accurateAudioStream = mp[5] & 0x02 ? 1 : 0;
 
   info->maxReadSpeed = (mp[8] << 8) | mp[9];
@@ -1671,7 +1694,7 @@ CdRawToc *GenericMMC::getRawToc(int sessionNr, int *len)
 
   dataLen = ((reqData[0] << 8) | reqData[1]) + 2;
   
-  message(3, "Raw toc data len: %d", dataLen);
+  message(4, "Raw toc data len: %d", dataLen);
 
   data = new (unsigned char)[dataLen];
   
@@ -1691,7 +1714,7 @@ CdRawToc *GenericMMC::getRawToc(int sessionNr, int *len)
 
   for (i = 0, p = data + 4; i < entries; i++, p += 11 ) {
 #if 0
-    message(0, "%d %02x %02d %2x %02d:%02d:%02d %02d %02d:%02d:%02d",
+    message(5, "%d %02x %02d %2x %02d:%02d:%02d %02d %02d:%02d:%02d",
 	    p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7], p[8], p[9], p[10]);
 #endif
     rawToc[i].sessionNr = p[0];
@@ -1756,10 +1779,18 @@ long GenericMMC::readTrackData(TrackData::Mode mode, long lba, long len,
 	    break;
 	  }
 	}
+	else if ((sense[2] & 0x0f) == 4) { // Hardware Error
+          switch (sense[12]) {
+          case 0x9: // focus servo failure
+            return -2;
+            break;
+          }
+        }    
 	else if ((sense[2] & 0x0f) == 3) { // Medium error
 	  switch (sense[12]) {
 	  case 0x02: // No seek complete, sector not found
 	  case 0x11: // L-EC error
+	  case 0x15: // random positioning error
 	    return -2;
 	    break;
 	  }
@@ -1786,17 +1817,15 @@ long GenericMMC::readTrackData(TrackData::Mode mode, long lba, long len,
   unsigned char *sector = transferBuffer_;
   for (i = 0; i < len; i++) {
      
-
     if (memcmp(CdrDriver::syncPattern, sector, 12) != 0) {
       // can't be a data block
-      message(3, "Stopped because no sync pattern found.");
-      message(3, "%d %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x", i, 
+      message(4, "Stopped because no sync pattern found.");
+      message(4, "%d %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x", i, 
 	      sector[0], sector[1], sector[2], sector[3], sector[4], sector[5],
 	      sector[6], sector[7], sector[8], sector[9], sector[10],
 	      sector[11]);
       return i;
     }
-
     actMode = determineSectorMode(sector + 12);
 
     if (!(actMode == mode ||
@@ -1810,9 +1839,9 @@ long GenericMMC::readTrackData(TrackData::Mode mode, long lba, long len,
 	   (actMode == TrackData::MODE2 ||
 	    actMode == TrackData::MODE2_FORM1 ||
 	    actMode == TrackData::MODE2_FORM2)))) {
-      message(3, "Sector with not matching mode %s found.",
+      message(4, "Sector with not matching mode %s found.",
 	      TrackData::mode2String(actMode));
-      message(3, "%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x",
+      message(4, "%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x",
 	      sector[12], sector[13], sector[14], sector[15], sector[16],
 	      sector[17], sector[18], sector[19], sector[20], sector[21],
 	      sector[22], sector[23]);
@@ -1860,7 +1889,8 @@ long GenericMMC::readTrackData(TrackData::Mode mode, long lba, long len,
   return len;
 }
 
-int GenericMMC::readAudioRange(int fd, long start, long end, int startTrack,
+int GenericMMC::readAudioRange(ReadDiskInfo *rinfo, int fd, long start,
+			       long end, int startTrack,
 			       int endTrack, TrackInfo *info)
 {
   if (!onTheFly_) {
@@ -1878,9 +1908,14 @@ int GenericMMC::readAudioRange(int fd, long start, long end, int startTrack,
 
 
       for (t = startTrack; t <= endTrack; t++) {
+	long totalProgress;
+
 	message(1, "Track %d...", t + 1);
 
-	sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 0);
+	totalProgress = t * 1000;
+	totalProgress /= rinfo->tracks;
+	sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 0,
+			      totalProgress);
 
 	if (options_ & OPT_MMC_NO_SUBCHAN) {
 	  // we have to use the binary search method to find pre-gap and
@@ -1892,7 +1927,7 @@ int GenericMMC::readAudioRange(int fd, long start, long end, int startTrack,
 	    unsigned char ctl;
 
 	    if (pregap > 0)
-	      message(1, "Found pre-gap: %s", Msf(pregap).str());
+	      message(2, "Found pre-gap: %s", Msf(pregap).str());
 
 	    slba = info[t].start;
 	    if (info[t].mode == info[t + 1].mode)
@@ -1924,16 +1959,19 @@ int GenericMMC::readAudioRange(int fd, long start, long end, int startTrack,
 	info[t].isrcCode[0] = 0;
 	readIsrc(t + 1, info[t].isrcCode);
 	if (info[t].isrcCode[0] != 0)
-	  message(1, "Found ISRC code.");
+	  message(2, "Found ISRC code.");
 
-	sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 1000);
+	totalProgress = (t + 1) * 1000;
+	totalProgress /= rinfo->tracks;
+	sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 1000,
+			      totalProgress);
       }
 
       message(1, "Reading...");
     }
   }
 
-  return CdrDriver::readAudioRangeParanoia(fd, start, end, startTrack,
+  return CdrDriver::readAudioRangeParanoia(rinfo, fd, start, end, startTrack,
 					   endTrack, info);
 }
 
