@@ -18,6 +18,23 @@
  */
 /*
  * $Log: PQChannelEncoder.h,v $
+ * Revision 1.3  2001/01/28 10:37:15  andreasm
+ * generic-mmc-raw: Fixed Q sub-channel encoding for lead-in regarding toc type
+ * and flags of data tracks.
+ * Fixed encoding of CD-TEXT packs into sub-channel. The last sub-channel is
+ * now always completely filled with valid CD-TEXT packs.
+ * Added driver options to define if the raw toc data contains BCD or HEX
+ * values so that the auto detection can be skipped.
+ * The 'blank' command now waits for completion. Added possibility to specify
+ * blanking mode (full, minimal).
+ * Updated man page, README and INSTALL.
+ *
+ * Revision 1.2  2000/12/17 10:51:22  andreasm
+ * Default verbose level is now 2. Adaopted message levels to have finer
+ * grained control about the amount of messages printed by cdrdao.
+ * Added CD-TEXT writing support to the GenericMMCraw driver.
+ * Fixed CD-TEXT cue sheet creating for the GenericMMC driver.
+ *
  * Revision 1.1.1.1  2000/02/05 01:35:04  llanero
  * Uploaded cdrdao 1.1.3 with pre10 patch applied.
  *
@@ -48,6 +65,25 @@ struct CueSheetEntry {
 
 class PQChannelEncoder {
 public:
+  struct SessionInfo {
+    // required for all sessions
+    int sessionNr;
+    long leadInStart;
+    long leadInLen;
+    long leadOutLen;
+    
+    // required for all sessions of a multi session disk
+    Msf lastLeadoutStart;
+
+    // required for 1st session of a multi session disk
+    int cdrw;
+    Msf atipLeadinStart;
+    unsigned char optimumRecordingPower;
+
+    // required for a CD-RW for 1st session of a multi session disk
+    unsigned char atipA1[3];
+  };
+
   PQChannelEncoder();
   ~PQChannelEncoder();
 
@@ -72,6 +108,10 @@ private:
   int lastTrackNr_;
   Msf leadOutStart_;
 
+  unsigned char firstTrackCtlAdr_;
+  unsigned char lastTrackCtlAdr_;
+  unsigned char leadOutCtlAdr_;
+
   CueSheetEntry *actCueSheetEntry_;
   long trlba_; // track relative lba
   long abslba_;  // absolute lba on disc
