@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998-2000  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2001  Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,52 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: PlextorReader.cc,v $
- * Revision 1.2  2000/04/23 16:29:50  andreasm
- * Updated to state of my private development environment.
- *
- * Revision 1.12  1999/12/15 20:31:46  mueller
- * Added remote messages for 'read-cd' progress used by a GUI.
- *
- * Revision 1.11  1999/12/12 13:37:33  mueller
- * Added DAE command for Matshita CD-ROM drives.
- *
- * Revision 1.10  1999/11/07 09:14:59  mueller
- * Release 1.1.3
- *
- * Revision 1.9  1999/04/05 18:48:37  mueller
- * Added driver options.
- * Added read-cd patch from Leon Woestenberg.
- *
- * Revision 1.8  1999/03/27 20:56:39  mueller
- * Changed toc analysis interface.
- * Added support for toc analysis of data disks.
- *
- * Revision 1.7  1998/09/27 19:20:05  mueller
- * Added retrieval of control nibbles for track with 'analyzeTrack()'.
- *
- * Revision 1.6  1998/09/08 11:54:22  mueller
- * Extended disk info structure because CDD2000 does not support the
- * 'READ DISK INFO' command.
- *
- * Revision 1.5  1998/09/07 15:20:20  mueller
- * Reorganized read-toc related code.
- *
- * Revision 1.4  1998/09/06 13:34:22  mueller
- * Use 'message()' for printing messages.
- *
- * Revision 1.3  1998/08/30 19:25:43  mueller
- * Added function 'diskInfo()'.
- * Changed sub channel data field in 'readSubChannelData()' to be more
- * compatible to other drives.
- *
- * Revision 1.2  1998/08/25 19:26:07  mueller
- * Moved basic index extraction algorithm to class 'CdrDriver'.
- *
- */
-
-static char rcsid[] = "$Id: PlextorReader.cc,v 1.2 2000/04/23 16:29:50 andreasm Exp $";
 
 #include <config.h>
 
@@ -118,8 +72,8 @@ PlextorReader::PlextorReader(ScsiIf *scsiIf, unsigned long options)
     }
     model_ = models[m].number; // zero if not found
 
-    message(3, "model number %d\n",model_);
-    message(3, "PRODUCT ID: '%s'\n", scsiIf_->product());
+    message(4, "model number %d\n",model_);
+    message(4, "PRODUCT ID: '%s'\n", scsiIf_->product());
   }
 }
 
@@ -375,7 +329,7 @@ CdRawToc *PlextorReader::getRawToc(int sessionNr, int *len)
 
   dataLen = ((reqData[0] << 8) | reqData[1]) + 2;
 
-  message(3, "Raw toc data len: %d", dataLen);
+  message(4, "Raw toc data len: %d", dataLen);
 
   data = new (unsigned char)[dataLen];
   
@@ -502,9 +456,9 @@ long PlextorReader::readTrackData(TrackData::Mode mode, long lba, long len,
 	   (actMode == TrackData::MODE2 ||
 	    actMode == TrackData::MODE2_FORM1 ||
 	    actMode == TrackData::MODE2_FORM2)))) {
-      message(3, "Stopped because sector with not matching mode %s found.",
+      message(4, "Stopped because sector with not matching mode %s found.",
 	      TrackData::mode2String(actMode));
-      message(3, "%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x",
+      message(4, "%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x",
 	      sector[0], sector[1], sector[2], sector[3],  sector[4],
 	      sector[5], sector[6], sector[7], sector[8], sector[9],
 	      sector[10], sector[11]);
@@ -571,7 +525,8 @@ Toc *PlextorReader::readDisk(int session, const char *dataFilename)
 // Creates 'Toc' object for inserted CD.
 // audioFilename: name of audio file that is placed into TOC
 // Return: newly allocated 'Toc' object or 'NULL' on error
-int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
+int PlextorReader::readAudioRangePlextor(ReadDiskInfo *rinfo, int fd,
+					 long startLba, long endLba,
 					 int startTrack, int endTrack, 
 					 TrackInfo *info)
 {
@@ -625,7 +580,7 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
 
   cat = fat + 1;
  
-  message(1, "Reading CDDA blocks in range [%lu,%lu>"
+  message(2, "Reading CDDA blocks in range [%lu,%lu>"
     " (%lu blocks=%lu bytes).",fab,lab,nab,nab*2352);
 
   memset(cmd,0,12);
@@ -671,6 +626,7 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
       if (cab > stampLba + 75) {
 	long totalLen = info[cat + 1].start - info[cat].start;
 	long progress = cab - info[cat].start;
+	long totalProgress;
 
 	if (progress > 0) {
 	  progress *= 1000;
@@ -680,7 +636,18 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
 	  progress = 0;
 	}
 
-	sendReadCdProgressMsg(RCD_EXTRACTING, cat + 1, progress);
+	totalProgress = cab - rinfo->startLba;
+
+	if (totalProgress > 0) {
+	  totalProgress *= 1000;
+	  totalProgress /= rinfo->endLba - rinfo->startLba;
+	}
+	else {
+	  totalProgress = 0;
+	}
+
+	sendReadCdProgressMsg(RCD_EXTRACTING, rinfo->tracks, cat + 1, progress,
+			      totalProgress);
 
 	stampLba = cab;
       }
@@ -730,7 +697,7 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
 
           if ((chan.indexNr()!=cai) || (chan.trackNr()!=cat))
           {
-            message(1, "(track,index,absolute)=(%2d,%2d,%s)",
+            message(2, "(track,index,absolute)=(%2d,%2d,%s)",
               chan.trackNr(),chan.indexNr(),qAbsTime.str());
 
             // first track start?
@@ -740,7 +707,7 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
 		      chan.trackNr(),qAbsTime.str(),cab);
               if ((qAbsTime.lba()-150)!=info[0].start)
               {
-                message(1, "TOC SAYS TRACK STARTS AT %s!",
+                message(2, "TOC SAYS TRACK STARTS AT %s!",
                   Msf(info[0].start).str());
               }
             }
@@ -751,16 +718,16 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
                 chan.trackNr(),qAbsTime.str());
               if ((qAbsTime.lba()-150)!=info[cat].start)
               {
-                message(1, "TOC SAYS TRACK STARTS AT %s!",
+                message(2, "TOC SAYS TRACK STARTS AT %s!",
 			Msf(info[cat].start).str());
               }
             }
             // pregap of next track?
             else if ((chan.trackNr()==(cat+1)) && (chan.indexNr()==0))
             {
-              message(1, "Found pregap of track %02d with size %s at ",
+              message(2, "Found pregap of track %02d with size %s at ",
 		      chan.trackNr(), qRelTime.str());
-	      message(1, "%s", qAbsTime.str());
+	      message(2, "%s", qAbsTime.str());
 	      if (cat <= lat) {
 		info[cat].pregap = qRelTime.lba();
 	      }
@@ -770,7 +737,7 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
             {
               if (chan.indexNr() > 1)
               {
-		message(1, "Found index number %02d at %s.",
+		message(2, "Found index number %02d at %s.",
 			chan.indexNr(),qAbsTime.str());
 
 		int indexCnt = info[cat - 1].indexCnt;
@@ -782,7 +749,7 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
             }
             else if (chan.trackNr()==0xAA)
             {
-              message(1, "LEAD OUT TRACK ENCOUNTERED!");
+              message(2, "LEAD OUT TRACK ENCOUNTERED!");
               n=0;
               i=0;
               rab=0;
@@ -806,7 +773,7 @@ int PlextorReader::readAudioRangePlextor(int fd,  long startLba, long endLba,
 	  // subchannel contains ISRC code
 	  if (info[cat - 1].isrcCode[0] == 0) {
 	    memcpy(info[cat - 1].isrcCode, chan.isrc(), 13);
-	    message(1, "Found ISRC code of track %02d.", cat);
+	    message(2, "Found ISRC code of track %02d.", cat);
 	  }
 	}
       }
@@ -1053,14 +1020,15 @@ int PlextorReader::readSubChannels(long lba, long len, SubChannel ***chans,
   return 0;
 }
 
-int PlextorReader::readAudioRange(int fd, long start, long end,
-				  int startTrack, int endTrack, 
+int PlextorReader::readAudioRange(ReadDiskInfo *rinfo, int fd, long start,
+				  long end, int startTrack, int endTrack, 
 				  TrackInfo *info)
 {
   if (model_ != 0 && (options_ & OPT_PLEX_USE_PARANOIA) == 0) {
     // we have a plextor drive -> use the special plextor method which
     // will also detect pre-gaps, index marks and ISRC codes
-    return readAudioRangePlextor(fd, start, end, startTrack, endTrack, info);
+    return readAudioRangePlextor(rinfo, fd, start, end, startTrack, endTrack,
+				 info);
   }
   
   if (!onTheFly_) {
@@ -1074,14 +1042,19 @@ int PlextorReader::readAudioRange(int fd, long start, long end,
     message(1, "Analyzing...");
     
     for (t = startTrack; t <= endTrack; t++) {
+      long totalProgress;
+
       message(1, "Track %d...", t + 1);
 
-      sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 0);
+      totalProgress = t * 1000;
+      totalProgress /= rinfo->tracks;
+      sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 0,
+			    totalProgress);
 
 
       if (!fastTocReading_) {
 	if (pregap != 0)
-	  message(1, "Found pre-gap: %s", Msf(pregap).str());
+	  message(2, "Found pre-gap: %s", Msf(pregap).str());
 
 	slba = info[t].start;
 	if (info[t].mode == info[t + 1].mode)
@@ -1111,14 +1084,18 @@ int PlextorReader::readAudioRange(int fd, long start, long end,
       info[t].isrcCode[0] = 0;
       readIsrc(t + 1, info[t].isrcCode);
       if (info[t].isrcCode[0] != 0)
-	message(1, "Found ISRC code.");
+	message(2, "Found ISRC code.");
 
-      sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 1000);
+      totalProgress = (t + 1) * 1000;
+      totalProgress /= rinfo->tracks;
+
+      sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 1000,
+			    totalProgress);
     }
 
     message(1, "Reading...");
   }
 
-  return CdrDriver::readAudioRangeParanoia(fd, start, end, startTrack,
+  return CdrDriver::readAudioRangeParanoia(rinfo, fd, start, end, startTrack,
 					   endTrack, info);
 }
