@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998-2000  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2001  Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,35 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: PlextorReaderScan.cc,v $
- * Revision 1.2  2000/04/23 16:29:50  andreasm
- * Updated to state of my private development environment.
- *
- * Revision 1.6  1999/12/15 20:31:46  mueller
- * Added remote messages for 'read-cd' progress used by a GUI.
- *
- * Revision 1.5  1999/11/07 09:14:59  mueller
- * Release 1.1.3
- *
- * Revision 1.4  1999/04/05 18:49:22  mueller
- * Added driver options.
- * Added option to read Q sub-channel data instead of raw PW sub-channel
- * data for 'read-toc'.
- *
- * Revision 1.3  1999/03/27 20:56:39  mueller
- * Changed toc analysis interface.
- * Added support for toc analysis of data disks.
- *
- * Revision 1.2  1998/09/27 19:20:05  mueller
- * Added retrieval of control nibbles for track with 'analyzeTrack()'.
- *
- * Revision 1.1  1998/09/07 15:15:40  mueller
- * Initial revision
- *
- */
-
-static char rcsid[] = "$Id: PlextorReaderScan.cc,v 1.2 2000/04/23 16:29:50 andreasm Exp $";
 
 #include <config.h>
 
@@ -224,8 +195,8 @@ int PlextorReaderScan::readSubChannels(long lba, long len, SubChannel ***chans,
   return 0;
 }
 
-int PlextorReaderScan::readAudioRange(int fd, long start, long end,
-				      int startTrack, int endTrack, 
+int PlextorReaderScan::readAudioRange(ReadDiskInfo *info, int fd, long start,
+				      long end, int startTrack, int endTrack, 
 				      TrackInfo *trackInfo)
 {
   if (!onTheFly_) {
@@ -236,21 +207,30 @@ int PlextorReaderScan::readAudioRange(int fd, long start, long end,
       message(1, "Analyzing...");
       
       for (t = startTrack; t <= endTrack; t++) {
+	long totalProgress;
+
 	message(1, "Track %d...", t + 1);
-	sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 0);
+
+	totalProgress = t * 1000;
+	totalProgress /= info->tracks;
+	sendReadCdProgressMsg(RCD_ANALYZING, info->tracks, t + 1, 0,
+			      totalProgress);
 
 	trackInfo[t].isrcCode[0] = 0;
 	readIsrc(t + 1, trackInfo[t].isrcCode);
 	if (trackInfo[t].isrcCode[0] != 0)
-	  message(1, "Found ISRC code.");
+	  message(2, "Found ISRC code.");
 
-	sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 1000);
+	totalProgress = (t + 1) * 1000;
+	totalProgress /= info->tracks;
+	sendReadCdProgressMsg(RCD_ANALYZING, info->tracks, t + 1, 1000,
+			      totalProgress);
       }
 
       message(1, "Reading...");
     }
   }
 
-  return CdrDriver::readAudioRangeParanoia(fd, start, end, startTrack,
+  return CdrDriver::readAudioRangeParanoia(info, fd, start, end, startTrack,
 					   endTrack, trackInfo);
 }
