@@ -18,6 +18,10 @@
  */
 /*
  * $Log: SonyCDU920.h,v $
+ * Revision 1.2  2000/10/08 16:39:40  andreasm
+ * Remote progress message now always contain the track relative and total
+ * progress and the total number of processed tracks.
+ *
  * Revision 1.1.1.1  2000/02/05 01:35:11  llanero
  * Uploaded cdrdao 1.1.3 with pre10 patch applied.
  *
@@ -93,8 +97,8 @@ protected:
   long readTrackData(TrackData::Mode mode, long lba, long len,
 		     unsigned char *buf);
 
-  int readAudioRange(int fd, long start, long end, int startTrack,
-		     int endTrack, TrackInfo *);
+  int readAudioRange(ReadDiskInfo *, int fd, long start, long end,
+		     int startTrack, int endTrack, TrackInfo *);
 
 };
 
