@@ -19,6 +19,16 @@
 
 /*
  * $Log: YamahaCDR10x.cc,v $
+ * Revision 1.4  2000/12/17 10:51:23  andreasm
+ * Default verbose level is now 2. Adaopted message levels to have finer
+ * grained control about the amount of messages printed by cdrdao.
+ * Added CD-TEXT writing support to the GenericMMCraw driver.
+ * Fixed CD-TEXT cue sheet creating for the GenericMMC driver.
+ *
+ * Revision 1.3  2000/10/08 16:39:41  andreasm
+ * Remote progress message now always contain the track relative and total
+ * progress and the total number of processed tracks.
+ *
  * Revision 1.2  2000/04/23 16:29:50  andreasm
  * Updated to state of my private development environment.
  *
@@ -60,7 +70,7 @@
  * Written by Cameron G. MacKinnon <C_MacKinnon@yahoo.com>.
  */
 
-static char rcsid[] = "$Id: YamahaCDR10x.cc,v 1.2 2000/04/23 16:29:50 andreasm Exp $";
+static char rcsid[] = "$Id: YamahaCDR10x.cc,v 1.4 2000/12/17 10:51:23 andreasm Exp $";
 
 #include <config.h>
 
@@ -456,11 +466,11 @@ unsigned char *YamahaCDR10x::createCueSheet(long *cueSheetLen)
   cueSheet[n*8+6] = SubChannel::bcd(lostart.sec());
   cueSheet[n*8+7] = SubChannel::bcd(lostart.frac());
 
-  message(3, "\nCue Sheet:");
-  message(3, "CTL/  TNO  INDEX  DATA  DATA  MIN  SEC  FRAME");
-  message(3, "ADR   BCD  BCD    TYPE  Form  BCD  BCD  BCD <- Note");
+  message(4, "\nCue Sheet:");
+  message(4, "CTL/  TNO  INDEX  DATA  DATA  MIN  SEC  FRAME");
+  message(4, "ADR   BCD  BCD    TYPE  Form  BCD  BCD  BCD <- Note");
   for (n = 0; n < len; n++) {
-    message(3, "%02x    %02x    %02x     %02x    %02x   %02x   %02x   %02x",
+    message(4, "%02x    %02x    %02x     %02x    %02x   %02x   %02x   %02x",
 	   cueSheet[n*8],
 	   cueSheet[n*8+1], cueSheet[n*8+2], cueSheet[n*8+3], cueSheet[n*8+4],
 	   cueSheet[n*8+5], cueSheet[n*8+6], cueSheet[n*8+7]);
@@ -556,7 +566,7 @@ int YamahaCDR10x::startDao()
 	return 1;
   }
 
-  //message(1, "Writing lead-in and gap...");
+  //message(2, "Writing lead-in and gap...");
 
   long lba = -150;
 
@@ -573,7 +583,7 @@ int YamahaCDR10x::finishDao()
   const unsigned char *sense;
   int senseLen;
 
-  message(1, "Flushing cache...");
+  message(2, "Flushing cache...");
   
   if (flushCache() != 0) {
     return 1;
@@ -1073,8 +1083,9 @@ Toc *YamahaCDR10x::readDisk(int session, const char *fname)
   return toc;
 }
 
-int YamahaCDR10x::readAudioRange(int fd, long start, long end,
-				 int startTrack, int endTrack, TrackInfo *info)
+int YamahaCDR10x::readAudioRange(ReadDiskInfo *rinfo, int fd, long start,
+				 long end, int startTrack,
+				 int endTrack, TrackInfo *info)
 {
   if (!onTheFly_) {
     int t;
@@ -1082,20 +1093,28 @@ int YamahaCDR10x::readAudioRange(int fd, long start, long end,
     message(1, "Analyzing...");
     
     for (t = startTrack; t <= endTrack; t++) {
-      sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 0);
+      long totalProgress;
+
+      totalProgress = t * 1000;
+      totalProgress /= rinfo->tracks;
+      sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 0,
+			    totalProgress);
 
       message(1, "Track %d...", t + 1);
       info[t].isrcCode[0] = 0;
       readIsrc(t + 1, info[t].isrcCode);
       if (info[t].isrcCode[0] != 0)
-	message(1, "Found ISRC code.");
+	message(2, "Found ISRC code.");
 
-      sendReadCdProgressMsg(RCD_ANALYZING, t + 1, 1000);
+      totalProgress = (t + 1) * 1000;
+      totalProgress /= rinfo->tracks;
+      sendReadCdProgressMsg(RCD_ANALYZING, rinfo->tracks, t + 1, 1000,
+			    totalProgress);
     }
 
     message(1, "Reading...");
   }
 
-  return CdrDriver::readAudioRangeParanoia(fd, start, end, startTrack,
+  return CdrDriver::readAudioRangeParanoia(rinfo, fd, start, end, startTrack,
 					   endTrack, info);
 }
