@@ -18,6 +18,23 @@
  */
 /*
  * $Log: dao.cc,v $
+ * Revision 1.7  2001/03/25 07:36:14  andreasm
+ * Updated SCSI lib to version from cdrtools-1.10a17.
+ * Added patches from compilation under UnixWare.
+ *
+ * Revision 1.6  2000/12/17 10:51:23  andreasm
+ * Default verbose level is now 2. Adaopted message levels to have finer
+ * grained control about the amount of messages printed by cdrdao.
+ * Added CD-TEXT writing support to the GenericMMCraw driver.
+ * Fixed CD-TEXT cue sheet creating for the GenericMMC driver.
+ *
+ * Revision 1.5  2000/11/12 16:50:44  andreasm
+ * Fixes for compilation under Win32.
+ *
+ * Revision 1.4  2000/10/08 16:39:41  andreasm
+ * Remote progress message now always contain the track relative and total
+ * progress and the total number of processed tracks.
+ *
  * Revision 1.3  2000/06/06 22:26:13  andreasm
  * Updated list of supported drives.
  * Added saving of some command line settings to $HOME/.cdrdao.
@@ -64,7 +81,7 @@
  *
  */
 
-static char rcsid[] = "$Id: dao.cc,v 1.3 2000/06/06 22:26:13 andreasm Exp $";
+static char rcsid[] = "$Id: dao.cc,v 1.7 2001/03/25 07:36:14 andreasm Exp $";
 
 #include <config.h>
 
@@ -91,16 +108,21 @@ static char rcsid[] = "$Id: dao.cc,v 1.3 2000/06/06 22:26:13 andreasm Exp $";
 
 #ifdef USE_POSIX_THREADS
 #include <pthread.h>
-#include <sched.h>
 #else
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #endif
 
+#include "dao.h"
+
 #include "port.h"
+#include "util.h"
+
 
 #define DEBUG_WRITE 0
 
+
+
 #if defined(__FreeBSD__)
 #define IPC_ARG_T void
 #else
@@ -108,39 +130,6 @@ static char rcsid[] = "$Id: dao.cc,v 1.3 2000/06/06 22:26:13 andreasm Exp $";
 #endif
 
 
-/* Select POSIX scheduler interface for real time scheduling if possible */
-#if (defined HAVE_SCHED_GETPARAM) && (defined HAVE_SCHED_GET_PRIORITY_MAX) && (defined HAVE_SCHED_SETSCHEDULER) && (!defined LINUX_QNX_SCHEDULING)
-#define POSIX_SCHEDULING
-#endif
-
-#ifdef LINUX_QNX_SCHEDULING
-
-#define SCHED_OTHER     0
-#define SCHED_FIFO      1
-#define SCHED_RR        2
-
-struct sched_param {
-  unsigned int priority;
-  int fork_penalty_threshold;
-  unsigned int starvation_threshold;
-  unsigned int ts_max;
-  unsigned int run_q, run_q_min, run_q_max;
-};
-extern "C" int sched_setparam __P((pid_t __pid, const struct sched_param *__param));
-extern "C" int sched_getparam __P((pid_t __pid, struct sched_param *__param));
-extern "C" int sched_setscheduler __P((pid_t __pid, int __policy, const struct sched_param *__param));
-extern "C" int sched_getscheduler __P((pid_t __pid));
-
-#elif defined POSIX_SCHEDULING
-
-#include <sched.h>
-
-#endif
-
-#include "dao.h"
-#include "util.h"
-#include "remote.h"
-
 struct ShmSegment {
   int id;
   char *buffer;
@@ -153,7 +142,7 @@ struct Buffer {
                              // blocks must be encoded in audio blocks,
                              // only used for message printing
   int trackNr; // if != 0 a new track with given number has started
-
+  int trackProgress; // reading progress of current track 0..1000
   char *buffer; // address of buffer that should be written
 };
   
@@ -276,9 +265,11 @@ static void unblockSignals()
 // return: 0: OK 
 //         1: child process terminated and has been collected with 'wait()'
 //         2: error -> child process must be terminated
-static int writer(CdrDriver *cdr, long total, BufferHeader *header, long lba,
-		  long remoteMode)
+static int writer(const Toc *toc, CdrDriver *cdr, BufferHeader *header,
+		  long lba)
 {
+  long total = toc->length().lba() * AUDIO_BLOCK_LEN;
+  long totalTracks = toc->nofTracks();
   long cnt = 0;
   long blkCount = 0;
   long len = 0;
@@ -291,17 +282,10 @@ static int writer(CdrDriver *cdr, long total, BufferHeader *header, long lba,
   int actTrackNr = 0;
   long actProgress;
   TrackData::Mode dataMode;
-  DaoWritingProgress remoteMsg;
-  char remoteMsgSync[4];
 #ifndef USE_POSIX_THREADS
   int status;
 #endif
 
-  remoteMsgSync[0] = 0xff;
-  remoteMsgSync[1] = 0x00;
-  remoteMsgSync[2] = 0xff;
-  remoteMsgSync[3] = 0x00;
-
   message (3, "Waiting for reader process");
 
   while (header->buffersFilled == 0) {
@@ -332,19 +316,8 @@ static int writer(CdrDriver *cdr, long total, BufferHeader *header, long lba,
   message (3, "Awaken, will start writing");
 
   if (cdr != NULL) {
-    if (remoteMode) {
-      remoteMsg.status = 1; // writing lead-in
-      remoteMsg.track = 0;
-      remoteMsg.totalProgress = 0;
-      remoteMsg.bufferFillRate = 100;
-
-      if (write(3/*fileno(stdout)*/, remoteMsgSync, sizeof(remoteMsgSync)) < 0 ||
-	  write(3/*fileno(stdout)*/, (const char*)&remoteMsg,
-		sizeof(remoteMsg)) < 0) {
-	message(-1, "Disabling remote messages.");
-	remoteMode = 0;
-      }
-    }
+    cdr->sendWriteCdProgressMsg(CdrDriver::WCD_LEADIN,
+				totalTracks, 0, 0, 0, 100);
 
     blockSignals();
     if (cdr->startDao() != 0) {
@@ -355,7 +328,7 @@ static int writer(CdrDriver *cdr, long total, BufferHeader *header, long lba,
   }
 
   do {
-    //message(3, "Slave: waiting for master.");
+    //message(4, "Slave: waiting for master.");
 
     while (header->buffersWritten == header->buffersRead) {
       if (header->readerTerminated) {
@@ -420,21 +393,10 @@ static int writer(CdrDriver *cdr, long total, BufferHeader *header, long lba,
 	fclose(fp);
 #endif
 
-      if (remoteMode) {
-	remoteMsg.status = 3; // writing lead-out
-	remoteMsg.track = 0xaa;
-	remoteMsg.totalProgress = 1000;
-	remoteMsg.bufferFillRate = 100;
-
-	if (write(3/*fileno(stdout)*/, remoteMsgSync, sizeof(remoteMsgSync)) < 0 ||
-	    write(3/*fileno(stdout)*/, (const char*)&remoteMsg,
-		  sizeof(remoteMsg)) < 0) {
-	  message(-1, "Disabling remote messages.");
-	  remoteMode = 0;
-	}
-      }
-
       if (cdr != NULL) {
+	cdr->sendWriteCdProgressMsg(CdrDriver::WCD_LEADOUT,
+				    totalTracks, 0xaa, 1000, 1000, 100);
+	
 	blockSignals();
 	if (cdr->finishDao() != 0) {
 	  unblockSignals();
@@ -457,7 +419,7 @@ static int writer(CdrDriver *cdr, long total, BufferHeader *header, long lba,
       actTrackNr = buf.trackNr;
     }
 
-    //message(3, "Slave: writing buffer %p (%ld).", buf, len);
+    //message(4, "Slave: writing buffer %p (%ld).", buf, len);
 
 #if DEBUG_WRITE
     if (fp != NULL) {
@@ -484,26 +446,15 @@ static int writer(CdrDriver *cdr, long total, BufferHeader *header, long lba,
 	}
       }
       unblockSignals();
-    }
-    else {
-      message(1, "Read %ld of %ld MB.\r", cnt >> 20, total >> 20);
-    }
 
-    if (remoteMode) {
       actProgress = cnt;
       actProgress /= total / 1000;
 
-      remoteMsg.status = 2; // writing data
-      remoteMsg.track = actTrackNr;
-      remoteMsg.totalProgress = actProgress;
-      remoteMsg.bufferFillRate = buffFill;
-      
-      if (write(3/*fileno(stdout)*/, remoteMsgSync, sizeof(remoteMsgSync)) < 0 ||
-	  write(3/*fileno(stdout)*/, (const char*)&remoteMsg,
-		sizeof(remoteMsg)) < 0) {
-	message(-1, "Disabling remote messages.");
-	remoteMode = 0;
-      }
+      cdr->sendWriteCdProgressMsg(CdrDriver::WCD_DATA, totalTracks, actTrackNr,
+				  buf.trackProgress, actProgress, buffFill);
+    }
+    else {
+      message(1, "Read %ld of %ld MB.\r", cnt >> 20, total >> 20);
     }
 
 
@@ -542,6 +493,10 @@ static void *reader(void *args)
   TrackData::Mode dataMode;
   int encodingMode = 1;
   int newTrack;
+  long tact; // number of blocks already read from current track
+  long tprogress;
+
+  setRealTimeScheduling(4);
 
   if (cdr != NULL) {
     if (cdr->bigEndianSamples() == 0) {
@@ -550,7 +505,7 @@ static void *reader(void *args)
     }
     encodingMode = cdr->encodingMode();
   }
-  message(3, "Swap: %d", swap);
+  message(4, "Swap: %d", swap);
 
   TrackIterator itr(toc);
   TrackReader reader;
@@ -562,7 +517,9 @@ static void *reader(void *args)
     message(-2, "Opening of track data failed.");
     goto fail;
   }
+
   newTrack = 1;
+  tact = 0;
 
   dataMode = (encodingMode == 0) ? TrackData::AUDIO : track->type();
 
@@ -592,10 +549,12 @@ static void *reader(void *args)
 	  dataMode = track->type();
 
 	newTrack = 1;
+	tact = 0;
       }
     } while (rn == 0);
 
     lba += rn;
+    tact += rn;
 
     if (track->type() == TrackData::AUDIO) {
       if (swap) {
@@ -610,11 +569,15 @@ static void *reader(void *args)
       }
     }
 
-    // notify writer that it can write buffer
     buf.bufLen = rn;
     buf.mode = dataMode;
     buf.trackMode = track->type();
 
+    tprogress = tact * 1000;
+    tprogress /= track->length().lba();
+
+    buf.trackProgress = tprogress;
+
     if (newTrack) {
       // inform write process that it should print message about new track
       buf.trackNr = trackNr;
@@ -637,7 +600,7 @@ static void *reader(void *args)
     }
     
     // wait for writing process to finish writing of previous buffer
-    //message(3, "Reader: waiting for Writer.");
+    //message(4, "Reader: waiting for Writer.");
     while (header->buffersRead - header->buffersWritten 
 	   == header->nofBuffers &&
 	   header->terminateReader == 0) {
@@ -679,21 +642,14 @@ fail:
 
 
 int writeDiskAtOnce(const Toc *toc, CdrDriver *cdr, int nofBuffers, int swap,
-		    int testMode, int remoteMode)
+		    int testMode)
 {
-  long length = toc->length().lba();
-  long total = length * AUDIO_BLOCK_LEN;
   int err = 0;
   BufferHeader *header = NULL;
   long nofShmSegments = 0;
   ShmSegment *shmSegments = NULL;
   long startLba = 0;
 
-#if defined(POSIX_SCHEDULING) || defined(LINUX_QNX_SCHEDULING) || \
-    defined(USE_POSIX_THREADS)
-  struct sched_param schedp;
-#endif
-
 #ifdef USE_POSIX_THREADS
   pthread_t readerThread;
   pthread_attr_t readerThreadAttr;
@@ -710,23 +666,6 @@ int writeDiskAtOnce(const Toc *toc, CdrDriver *cdr, int nofBuffers, int swap,
   }
 #endif
 
-  if (remoteMode) {
-    // switch stdout to non blocking IO
-    int flags;
-
-    if ((flags = fcntl(3/*fileno(stdout)*/, F_GETFL)) == -1) {
-      message(-2, "Cannot get flags of stdout: %s", strerror(errno));
-      return 1;
-    }
-
-    flags |= O_NONBLOCK;
-
-    if (fcntl(3/*fileno(stdout)*/, F_SETFL, flags) < 0) {
-      message(-2, "Cannot set flags of stdout: %s", strerror(errno));
-      return 1;
-    }
-  }
-
   if (getSharedMemory(nofBuffers, &header, &nofShmSegments,
 		      &shmSegments)  != 0) {
     releaseSharedMemory(nofShmSegments, shmSegments);
@@ -769,29 +708,6 @@ int writeDiskAtOnce(const Toc *toc, CdrDriver *cdr, int nofBuffers, int swap,
     err = 1; goto fail;
   }
   
-
-#if defined(POSIX_SCHEDULING) && defined(HAVE_PTHREAD_ATTR_SETSCHEDPOLICY) && \
-  defined(HAVE_PTHREAD_ATTR_SETSCHEDPARAM)
-
-  if (geteuid() == 0) {
-    if (pthread_attr_setschedpolicy(&readerThreadAttr, SCHED_RR) == 0) {
-      struct sched_param schedParam;
-
-      if (pthread_attr_getschedparam(&readerThreadAttr, &schedParam) == 0) {
-	schedParam.sched_priority = sched_get_priority_max(SCHED_RR) - 4;
-	if (pthread_attr_setschedparam(&readerThreadAttr, &schedParam) != 0) {
-	  message(-1, "pthread_attr_setschedparam failed: %s",
-		  strerror(errno));
-	}
-      }
-    }
-  }
-#else
-
-  message(-1, "Real time scheduling is not available for reader thread.");
-
-#endif
-
   ReaderArgs rargs;
 
   rargs.toc = toc;
@@ -807,8 +723,6 @@ int writeDiskAtOnce(const Toc *toc, CdrDriver *cdr, int nofBuffers, int swap,
   }
   else {
     threadStarted = 1;
-
-    
   }
 
 #else /* USE_POSIX_THREADS */
@@ -818,34 +732,12 @@ int writeDiskAtOnce(const Toc *toc, CdrDriver *cdr, int nofBuffers, int swap,
 
     setsid(); // detach from controlling terminal
 
-#ifdef LINUX_QNX_SCHEDULING
-
-    if (geteuid() == 0) {
-      sched_getparam (0, &schedp);
-      schedp.run_q_min = schedp.run_q_max = 1;
-      if (sched_setscheduler (0, SCHED_RR, &schedp) != 0) {
-	message(-1, "Cannot setup real time scheduling: %s", strerror(errno));
-      }
-    }
-
-#elif defined POSIX_SCHEDULING
-
-    if (geteuid() == 0) {
-      sched_getparam (0, &schedp);
-      schedp.sched_priority = sched_get_priority_max (SCHED_RR) - 4;
-      if (sched_setscheduler (0, SCHED_RR, &schedp) != 0) {
-	message(-1, "Cannot setup real time scheduling: %s", strerror(errno));
-      }
-    }
-
-#endif
-
 #ifdef HAVE_MLOCKALL
     if (geteuid() == 0) {
       if (mlockall(MCL_CURRENT|MCL_FUTURE) != 0) {
 	message(-1, "Cannot lock memory pages: %s", strerror(errno));
       }
-      message(3, "Reader process memory locked");
+      message(4, "Reader process memory locked");
     }
 #endif
 
@@ -865,74 +757,25 @@ int writeDiskAtOnce(const Toc *toc, CdrDriver *cdr, int nofBuffers, int swap,
   }
 #endif /* USE_POSIX_THREADS */
 
-  // setup scheduler for writing process/thread
-#if defined(USE_POSIX_THREADS)
-
-#if defined(HAVE_PTHREAD_GETSCHEDPARAM) && defined(HAVE_PTHREAD_SETSCHEDPARAM)
-  if (geteuid() == 0) {
-    int schedPolicy;
-
-    pthread_getschedparam(pthread_self(), &schedPolicy, &schedp);
-    schedp.sched_priority = sched_get_priority_max(SCHED_RR) - 5;
-
-    if (pthread_setschedparam(pthread_self(), SCHED_RR, &schedp) != 0) {
-      message(-1, "Cannot setup real time scheduling: %s", strerror(errno));
-    }
-    else {
-      message(1, "Using POSIX real time scheduling.");
-    }
+  switch (setRealTimeScheduling(5)) {
+  case 1:
+    message(-1, "No super user permission to setup real time scheduling.");
+    break;
+  case 2:
+    message(2, "Real time scheduling not available.");
+    break;
   }
-  else {
-    message(-1, "No root permissions for real time scheduling.");
-  }
-#else
-  message(-1, "Real time scheduling is not available.");
-#endif
-
-#elif defined(LINUX_QNX_SCHEDULING)
-  
-  if (geteuid() == 0) {
-    sched_getparam (0, &schedp);
-    schedp.run_q_min = schedp.run_q_max = 2;
-    if (sched_setscheduler (0, SCHED_RR, &schedp) != 0) {
-      message(-1, "Cannot setup real time scheduling: %s", strerror(errno));
-    }
-    else {
-      message(1, "Using Linux QNX real time scheduling.");
-    }
-  }
-  else {
-    message(-1, "No root permissions for real time scheduling.");
-  }
-
-#elif defined POSIX_SCHEDULING
-  
-  if (geteuid() == 0) {
-    sched_getparam (0, &schedp);
-    schedp.sched_priority = sched_get_priority_max (SCHED_RR) - 5;
-    if (sched_setscheduler (0, SCHED_RR, &schedp) != 0) {
-      message(-1, "Cannot setup real time scheduling: %s", strerror(errno));
-    }
-    else {
-      message(1, "Using POSIX real time scheduling.");
-    }
-  }
-  else {
-    message(-1, "No root permissions for real time scheduling.");
-  }
-  
-#endif
 
 #ifdef HAVE_MLOCKALL
   if (geteuid() == 0) {
     if (mlockall(MCL_CURRENT|MCL_FUTURE) != 0) {
       message(-1, "Cannot lock memory pages: %s", strerror(errno));
     }
-    message(3, "Memory locked");
+    message(4, "Memory locked");
   }
 #endif
 
-  switch (writer(cdr, total, header, startLba, remoteMode)) {
+  switch (writer(toc, cdr, header, startLba)) {
   case 1: // error, reader process terminated abnormally
 #ifndef USE_POSIX_THREADS
     pid = 0;
@@ -1080,7 +923,7 @@ static int getSharedMemory(long nofBuffers,
   maxSegmentSize = 1 * 1024 * 1024; // 1 MB
 #endif
 
-  message(3, "Shm max segement size: %ld (%ld MB)", maxSegmentSize,
+  message(4, "Shm max segement size: %ld (%ld MB)", maxSegmentSize,
 	  maxSegmentSize >> 20);
 
   if (maxSegmentSize < sizeof(BufferHeader) + nofBuffers * sizeof(Buffer)) {
@@ -1104,7 +947,7 @@ static int getSharedMemory(long nofBuffers,
 
   *shmSegments = new ShmSegment[*nofSegments];
 
-  message(3, "Using %ld shared memory segments.", *nofSegments);
+  message(4, "Using %ld shared memory segments.", *nofSegments);
 
   for (i = 0; i < *nofSegments; i++) {
     (*shmSegments)[i].id = -1;
@@ -1125,7 +968,7 @@ static int getSharedMemory(long nofBuffers,
       segmentLength += sizeof(BufferHeader) + nofBuffers * sizeof(Buffer);
     }
 
-    (*shmSegments)[i].id = shmget(IPC_PRIVATE, segmentLength, 0600);
+    (*shmSegments)[i].id = shmget(IPC_PRIVATE, segmentLength, 0600|IPC_CREAT);
     if ((*shmSegments)[i].id < 0) {
       message(-2, "Cannot create shared memory segment: %s",
 	      strerror(errno));
