@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998, 1999 Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2001 Andreas Mueller <andreas@daneb.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,45 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: Toc.cc,v $
- * Revision 1.2  2000/06/10 14:44:47  andreasm
- * Tracks that are shorter than 4 seconds do not lead to a fatal error anymore.
- * The user has the opportunity to record such tracks now.
- *
- * Revision 1.1.1.1  2000/02/05 01:32:54  llanero
- * Uploaded cdrdao 1.1.3 with pre10 patch applied.
- *
- * Revision 1.11  1999/04/05 11:03:01  mueller
- * Added CD-TEXT support.
- *
- * Revision 1.10  1999/04/02 20:36:21  mueller
- * Created implementation class that contains all mutual member data.
- *
- * Revision 1.9  1999/03/27 19:52:26  mueller
- * Added data track support.
- *
- * Revision 1.8  1999/01/10 15:10:13  mueller
- * Added functions 'appendTrack()' and 'appendAudioData()'.
- *
- * Revision 1.7  1998/11/15 12:19:13  mueller
- * Added several functions for manipulating track/index marks.
- *
- * Revision 1.6  1998/10/24 14:28:16  mueller
- * Fixed bug in 'readSamples()'.
- *
- * Revision 1.5  1998/10/03 14:32:31  mueller
- * Applied patch from Bjoern Fischer <bfischer@Techfak.Uni-Bielefeld.DE>.
- *
- * Revision 1.4  1998/09/22 19:17:19  mueller
- * Added seeking to and reading of samples for GUI.
- *
- * Revision 1.3  1998/09/06 12:00:26  mueller
- * Used 'message' function for messages.
- *
- */
-
-static char rcsid[] = "$Id: Toc.cc,v 1.2 2000/06/10 14:44:47 andreasm Exp $";
 
 #include <config.h>
 
@@ -70,6 +31,12 @@ static char rcsid[] = "$Id: Toc.cc,v 1.2 2000/06/10 14:44:47 andreasm Exp $";
 #include "TrackDataList.h"
 #include "CdTextItem.h"
 
+#ifdef UNIXWARE
+extern "C" {
+  extern int      strcasecmp(const char *, const char *);
+}
+#endif
+
 extern Toc *parseToc(FILE *fp, const char *filename);
 extern Toc *parseCue(FILE *fp, const char *filename);
 
@@ -1052,6 +1019,49 @@ int Toc::checkCdTextData() const
   return err;
 }
 
+
+void Toc::trackSummary(int *nofAudioTracks, int *nofMode1Tracks,
+		       int *nofMode2Tracks) const
+{
+  TrackEntry *run;
+
+  if (nofAudioTracks != NULL)
+    *nofAudioTracks = 0;
+
+  if (nofMode1Tracks != NULL)
+    *nofMode1Tracks = 0;
+
+  if (nofMode2Tracks != NULL)
+    *nofMode2Tracks = 0;
+
+  for (run = tracks_; run != NULL; run = run->next) {
+    switch (run->track->type()) {
+    case TrackData::AUDIO:
+      if (nofAudioTracks != NULL)
+	*nofAudioTracks += 1;
+      break;
+
+    case TrackData::MODE1:
+    case TrackData::MODE1_RAW:
+      if (nofMode1Tracks != NULL)
+	*nofMode1Tracks += 1;
+      break;
+
+    case TrackData::MODE2:
+    case TrackData::MODE2_FORM1:
+    case TrackData::MODE2_FORM2:
+    case TrackData::MODE2_FORM_MIX:
+    case TrackData::MODE2_RAW:
+      if (nofMode2Tracks != NULL)
+	*nofMode2Tracks += 1;
+      break;
+      
+    case TrackData::MODE0:
+      break;
+    }
+  }
+}
+
 // Class TrackIterator
 TrackIterator::TrackIterator(const Toc *t)
 {
