@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998, 1999  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2001  Andreas Mueller <andreas@daneb.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,39 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: Track.cc,v $
- * Revision 1.2  2000/06/10 14:44:47  andreasm
- * Tracks that are shorter than 4 seconds do not lead to a fatal error anymore.
- * The user has the opportunity to record such tracks now.
- *
- * Revision 1.1.1.1  2000/02/05 01:34:25  llanero
- * Uploaded cdrdao 1.1.3 with pre10 patch applied.
- *
- * Revision 1.9  1999/04/05 11:03:01  mueller
- * Added CD-TEXT support.
- *
- * Revision 1.8  1999/04/02 20:36:21  mueller
- * Created implementation class that contains all mutual member data.
- *
- * Revision 1.7  1999/03/27 19:52:26  mueller
- * Added data track support.
- *
- * Revision 1.6  1999/01/10 15:09:47  mueller
- * Fixed bug in sub-track list.
- *
- * Revision 1.5  1998/11/15 12:19:13  mueller
- * Added several functions for manipulating track/index marks.
- *
- * Revision 1.4  1998/09/22 19:17:19  mueller
- * Added seeking to and reading of samples for GUI.
- *
- * Revision 1.3  1998/08/15 17:58:13  mueller
- * Added missing printing of index increment data in 'print()'.
- *
- */
-
-static char rcsid[] = "$Id: Track.cc,v 1.2 2000/06/10 14:44:47 andreasm Exp $";
 
 #include <config.h>
 
@@ -1766,3 +1733,38 @@ long TrackReader::readSamples(Sample *buf, long len)
   
   return ret;
 }
+
+
+SubTrackIterator::SubTrackIterator(const Track *t)
+{
+  track_ = t;
+  iterator_ = NULL;
+}
+
+
+SubTrackIterator::~SubTrackIterator()
+{
+  track_ = NULL;
+  iterator_ = NULL;
+}
+
+const SubTrack *SubTrackIterator::first()
+{
+  iterator_ = track_->subTracks_;
+
+  return next();
+}
+
+const SubTrack *SubTrackIterator::next()
+{
+  if (iterator_ != NULL) {
+    SubTrack *s = iterator_;
+
+    iterator_ = iterator_->next_;
+    return s;
+  }
+  else {
+    return NULL;
+  }
+}
+
