@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998-2000 Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2001 Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,50 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: CdDevice.cc,v $
- * Revision 1.9  2000/08/01 01:27:50  llanero
- * CD to CD copy works now.
- *
- * Revision 1.8  2000/07/31 01:55:49  llanero
- * got rid of old Extract dialog and Record dialog.
- * both are using RecordProgressDialog now.
- *
- * Revision 1.7  2000/07/30 14:25:53  llanero
- * fixed bug with --device not receiving the right device
- *
- * Revision 1.6  2000/07/30 02:41:03  llanero
- * started CD to CD copy. Still not functional.
- *
- * Revision 1.5  2000/05/01 18:15:00  andreasm
- * Switch to gnome-config settings.
- * Adapted Message Box to Gnome look, unfortunately the Gnome::MessageBox is
- * not implemented in gnome--, yet.
- *
- * Revision 1.4  2000/04/29 14:46:38  llanero
- * added the "buffers" option to the Record Dialog.
- *
- * Revision 1.3  2000/04/28 19:08:10  llanero
- * modified glade files.
- * modified toolbar a little.
- * extract dialog has more option, and now you can specify the paranoia mode.
- *
- * Revision 1.2  2000/04/24 12:49:06  andreasm
- * Changed handling or message from remote processes to use the
- * Gtk::Main::input mechanism.
- *
- * Revision 1.3  1999/12/15 20:34:18  mueller
- * CD image extraction added by Manuel Clos.
- *
- * Revision 1.2  1999/11/07 09:18:45  mueller
- * Release 1.1.3
- *
- * Revision 1.1  1999/09/03 16:05:14  mueller
- * Initial revision
- *
- */
-
-static char rcsid[] = "$Id: CdDevice.cc,v 1.9 2000/08/01 01:27:50 llanero Exp $";
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -79,7 +35,7 @@ static char rcsid[] = "$Id: CdDevice.cc,v 1.9 2000/08/01 01:27:50 llanero Exp $"
 #include "ProcessMonitor.h"
 #include "xcdrdao.h"
 #include "guiUpdate.h"
-#include "RecordProgressDialog.h"
+#include "ProgressDialog.h"
 #include "Settings.h"
 
 #include "remote.h"
@@ -89,7 +45,7 @@ static char rcsid[] = "$Id: CdDevice.cc,v 1.9 2000/08/01 01:27:50 llanero Exp $"
 #include "util.h"
 #include "Toc.h"
 
-#define DRIVER_IDS 12
+#define DRIVER_IDS 13
 
 CdDevice *CdDevice::DEVICE_LIST_ = NULL;
 
@@ -105,6 +61,7 @@ char *CdDevice::DRIVER_NAMES_[DRIVER_IDS] = {
   "sony-cdu948",
   "taiyo-yuden",
   "teac-cdr55",
+  "toshiba",
   "yamaha-cdr10x"
 };
   
@@ -133,8 +90,10 @@ CdDevice::CdDevice(int bus, int id, int lun, const char *vendor,
 
   progressStatusChanged_ = 0;
   progressStatus_ = 0;
+  progressTotalTracks_ = 0;
   progressTrack_ = 0;
   progressTotal_ = 0;
+  progressTrackRelative_ = 0;
   progressBufferFill_ = 0;
 
   process_ = NULL;
@@ -143,6 +102,7 @@ CdDevice::CdDevice(int bus, int id, int lun, const char *vendor,
   scsiIfInitFailed_ = 0;
 
   next_ = NULL;
+  slaveDevice_ = NULL;
 
   autoSelectDriver();
 }
@@ -340,6 +300,11 @@ int CdDevice::updateStatus()
 
       PROCESS_MONITOR->remove(process_);
       process_ = NULL;
+
+      if (slaveDevice_ != NULL) {
+	slaveDevice_->status(DEV_UNKNOWN);
+	slaveDevice_ = NULL;
+      }
     }
   }
 
@@ -427,51 +392,30 @@ void CdDevice::updateProgress(int fd, GdkInputCondition cond)
       }
     }
 
-    if (status_ == DEV_RECORDING) {
-      DaoWritingProgress msg;
+    ProgressMsg msg;
 
-      if (read(fd, (char *)&msg, sizeof(msg)) == sizeof(msg)) {
-	/*
-	  message(0, "progress: %d %d %d %d", msg.status, msg.track,
-	  msg.totalProgress, msg.bufferFillRate);
-	  */
-	if (msg.status >= 1 && msg.status <= 3 &&
-	    msg.track >= 0 && msg.track <= 0xaa &&
-	    msg.totalProgress >= 0 && msg.totalProgress <= 1000 &&
-	    msg.bufferFillRate >= 0 && msg.bufferFillRate <= 100) {
-	  progressStatus_ = msg.status;
-	  progressTrack_ = msg.track;
-	  progressTotal_ = msg.totalProgress;
-	  progressBufferFill_ = msg.bufferFillRate;
-
-	  progressStatusChanged_ = 1;
-	}
-      }
-      else {
-	message(-1, "Reading of record progress msg failed.");
+    if (read(fd, (char *)&msg, sizeof(msg)) == sizeof(msg)) {
+      /*
+	message(0, "progress: %d %d %d %d", msg.status, msg.track,
+	msg.totalProgress, msg.bufferFillRate);
+      */
+      if (msg.status >= PGSMSG_MIN && msg.status <= PGSMSG_MAX &&
+	  msg.track >= 0 && msg.track <= 0xaa &&
+	  msg.trackProgress >= 0 && msg.trackProgress <= 1000 &&
+	  msg.totalProgress >= 0 && msg.totalProgress <= 1000 &&
+	  msg.bufferFillRate >= 0 && msg.bufferFillRate <= 100) {
+	progressStatus_ = msg.status;
+	progressTotalTracks_ = msg.totalTracks;
+	progressTrack_ = msg.track;
+	progressTrackRelative_ = msg.trackProgress;
+	progressTotal_ = msg.totalProgress;
+	progressBufferFill_ = msg.bufferFillRate;
+	
+	progressStatusChanged_ = 1;
       }
     }
-    else if (status_ == DEV_READING) {
-      ReadCdProgress msg;
-
-      if (read(fd, (char *)&msg, sizeof(msg)) == sizeof(msg)) {
-	/*
-	  message(0, "progress: %d %d %d %d", msg.status, msg.track,
-	  msg.totalProgress, msg.bufferFillRate);
-	  */
-	if (msg.status >= 1 && msg.status <= 2 &&
-	    msg.track >= 0 && msg.track <= 99 &&
-	    msg.trackProgress >= 0 && msg.trackProgress <= 1000) {
-	  progressStatus_ = msg.status;
-	  progressTrack_ = msg.track;
-	  progressTrackRelative_ = msg.trackProgress;
-
-	  progressStatusChanged_ = 1;
-	}
-      }
-      else {
-	message(-1, "Reading of record progress msg failed.");
-      }
+    else {
+      message(-1, "Reading of progress message failed.");
     }
   }
 
@@ -516,6 +460,7 @@ int CdDevice::recordDao(TocEdit *tocEdit, int simulate, int multiSession,
   char *execName;
   const char *s;
   char bufferbuf[20];
+  int remoteFdArgNum = 0;
 
   if (status_ != DEV_READY || process_ != NULL)
     return 1;
@@ -545,6 +490,9 @@ int CdDevice::recordDao(TocEdit *tocEdit, int simulate, int multiSession,
 
   args[n++] = "--remote";
 
+  remoteFdArgNum = n;
+  args[n++] = NULL;
+
   args[n++] = "-v0";
 
   if (multiSession)
@@ -590,21 +538,14 @@ int CdDevice::recordDao(TocEdit *tocEdit, int simulate, int multiSession,
   
   assert(n <= 20);
   
-  int i;
-
-  message(0, "Starting: ");
-  for (i = 0; i < n - 1; i++)
-    message(0, "%s ", args[i]);
-  message(0, "");
-
-  RECORD_PROGRESS_POOL->start(this, tocEdit);
+  PROGRESS_POOL->start(this, tocEdit);
 
   // Remove the SCSI interface of this device to avoid problems with double
   // usage of device nodes.
   delete scsiIf_;
   scsiIf_ = NULL;
 
-  process_ = PROCESS_MONITOR->start(execName, args);
+  process_ = PROCESS_MONITOR->start(execName, args, remoteFdArgNum);
 
   delete[] execName;
 
@@ -646,11 +587,14 @@ int CdDevice::progressStatusChanged()
   return 0;
 }
 
-void CdDevice::recordProgress(int *status, int *track, int *totalProgress,
-		      int *bufferFill) const
+void CdDevice::progress(int *status, int *totalTracks, int *track,
+			int *trackProgress, int *totalProgress,
+			int *bufferFill) const
 {
   *status = progressStatus_;
+  *totalTracks = progressTotalTracks_;
   *track = progressTrack_;
+  *trackProgress = progressTrackRelative_;
   *totalProgress = progressTotal_;
   *bufferFill = progressBufferFill_;
 }
@@ -658,16 +602,17 @@ void CdDevice::recordProgress(int *status, int *track, int *totalProgress,
 // Starts a 'cdrdao' for reading whole cd.
 // Return: 0: OK, process succesfully launched
 //         1: error occured
-int CdDevice::extractDao(char *tocFileName, int correction)
+int CdDevice::extractDao(const char *tocFileName, int correction)
 {
   char *args[20];
   int n = 0;
   char devname[30];
   char drivername[50];
-  char speedbuf[20];
+  //char speedbuf[20];
   char *execName;
   const char *s; 
   char correctionbuf[20];
+  int remoteFdArgNum = 0;
 
   if (status_ != DEV_READY || process_ != NULL)
     return 1;
@@ -684,6 +629,9 @@ int CdDevice::extractDao(char *tocFileName, int correction)
 
   args[n++] = "--remote";
 
+  remoteFdArgNum = n;
+  args[n++] = NULL;
+
   args[n++] = "-v0";
 
   args[n++] = "--read-raw";
@@ -717,21 +665,14 @@ int CdDevice::extractDao(char *tocFileName, int correction)
   
   assert(n <= 20);
   
-  int i;
-
-  message(0, "Starting: ");
-  for (i = 0; i < n - 1; i++)
-    message(0, "%s ", args[i]);
-  message(0, "");
-
-  RECORD_PROGRESS_POOL->start(this, tocFileName);
+  PROGRESS_POOL->start(this, tocFileName);
 
   // Remove the SCSI interface of this device to avoid problems with double
   // usage of device nodes.
   delete scsiIf_;
   scsiIf_ = NULL;
 
-  process_ = PROCESS_MONITOR->start(execName, args);
+  process_ = PROCESS_MONITOR->start(execName, args, remoteFdArgNum);
 
   delete[] execName;
 
@@ -760,20 +701,13 @@ void CdDevice::abortDaoReading()
   }
 }
 
-void CdDevice::readProgress(int *status, int *track, int *trackProgress) const
-{
-  *status = progressStatus_;
-  *track = progressTrack_;
-  *trackProgress = progressTrackRelative_;
-}
-
 // Starts a 'cdrdao' for duplicating a CD.
 // Return: 0: OK, process succesfully launched
 //         1: error occured
 int CdDevice::duplicateDao(int simulate, int multiSession, int speed,
-		int eject, int reload, int buffer, int onthefly, int correction, CdDevice *readdev)
+			   int eject, int reload, int buffer, int onthefly,
+			   int correction, CdDevice *readdev)
 {
-  char *tocFileName;
   char *args[25];
   int n = 0;
   char devname[30];
@@ -785,7 +719,7 @@ int CdDevice::duplicateDao(int simulate, int multiSession, int speed,
   char *execName;
   const char *s;
   char bufferbuf[20];
-
+  int remoteFdArgNum = 0;
 
 
   if (readdev->status() != DEV_READY || readdev->process() != NULL)
@@ -808,6 +742,9 @@ int CdDevice::duplicateDao(int simulate, int multiSession, int speed,
 
   args[n++] = "--remote";
 
+  remoteFdArgNum = n;
+  args[n++] = NULL;
+
   args[n++] = "-v0";
 
   if (multiSession)
@@ -876,38 +813,27 @@ int CdDevice::duplicateDao(int simulate, int multiSession, int speed,
   
   assert(n <= 25);
   
-  int i;
-
-  message(0, "Starting: ");
-  for (i = 0; i < n - 1; i++)
-    message(0, "%s ", args[i]);
-  message(0, "");
-
-
-  RECORD_PROGRESS_POOL->start(this, "CD to CD copy");
+  PROGRESS_POOL->start(this, "CD to CD copy");
 
   // Remove the SCSI interface of this device to avoid problems with double
   // usage of device nodes.
   delete scsiIf_;
   scsiIf_ = NULL;
 
-  process_ = PROCESS_MONITOR->start(execName, args);
+  process_ = PROCESS_MONITOR->start(execName, args, remoteFdArgNum);
 
   delete[] execName;
 
 
   if (process_ != NULL) {
-// don't do it for now!    readdev->status(DEV_READING);
-//    if (onthefly)
-      status_ = DEV_RECORDING;
-//    else
-//      status_ = DEV_WAITING;
+    slaveDevice_ = readdev;
+    slaveDevice_->status(DEV_READING);
+    status_ = DEV_RECORDING;
 
     action_ = A_DUPLICATE;
 
     if (process_->commFd() >= 0) {
-      Gtk::Main::instance()->input.connect(slot(this,
-						&CdDevice::updateProgress),
+      Gtk::Main::instance()->input.connect(slot(this, &CdDevice::updateProgress),
 					   process_->commFd(),
 					   (GdkInputCondition)(GDK_INPUT_READ|GDK_INPUT_EXCEPTION));
     }
@@ -1299,7 +1225,8 @@ void CdDevice::remove(int bus, int id, int lun)
   for (pred = NULL, run = DEVICE_LIST_; run != NULL;
        pred = run, run = run->next_) {
     if (bus == run->bus() && id == run->id() && lun == run->lun()) {
-      if (run->status() == DEV_RECORDING || run->status() == DEV_BLANKING)
+      if (run->status() == DEV_RECORDING || run->status() == DEV_BLANKING ||
+	  run->status() == DEV_READING || run->status() == DEV_WAITING)
 	return;
 	
       if (pred != NULL)
