@@ -18,6 +18,20 @@
  */
 /*
  * $Log: DeviceConfDialog.h,v $
+ * Revision 1.5  2000/10/01 16:39:10  llanero
+ * applied Jason Lunz patch: "Close" instead of "Cancel" where appropiate.
+ *
+ * Revision 1.4  2000/09/21 02:07:06  llanero
+ * MDI support:
+ * Splitted AudioCDChild into same and AudioCDView
+ * Move Selections from TocEdit to AudioCDView to allow
+ *   multiple selections.
+ * Cursor animation in all the views.
+ * Can load more than one from from command line
+ * Track info, Toc info, Append/Insert Silence, Append/Insert Track,
+ *   they all are built for every child when needed.
+ * ...
+ *
  * Revision 1.3  2000/04/23 09:07:08  andreasm
  * * Fixed most problems marked with '//llanero'.
  * * Added audio CD edit menus to MDIWindow.
@@ -53,10 +67,10 @@ public:
   DeviceConfDialog();
   ~DeviceConfDialog();
 
-  void start(TocEdit *);
+  void start();
   void stop();
 
-  void update(unsigned long level, TocEdit *);
+  void update(unsigned long level);
 
   gint delete_event_impl(GdkEventAny*);
 
@@ -93,7 +107,7 @@ private:
   void selectRow(gint, gint, GdkEvent *);
   void unselectRow(gint, gint, GdkEvent *);
 
-  void cancelAction();
+  void closeAction();
   void resetAction();
   void applyAction();
   void addDeviceAction();
