@@ -18,6 +18,24 @@
  */
 /*
  * $Log: SampleDisplay.cc,v $
+ * Revision 1.10  2001/04/29 12:10:27  andreasm
+ * Removed debug messages.
+ *
+ * Revision 1.9  2001/01/21 13:46:11  andreasm
+ * 'update()' functions of all dialogs require a 'TocEditView' object now.
+ * CD TEXT table entry is now a non modal dialog on its own.
+ *
+ * Revision 1.8  2000/09/21 02:07:06  llanero
+ * MDI support:
+ * Splitted AudioCDChild into same and AudioCDView
+ * Move Selections from TocEdit to AudioCDView to allow
+ *   multiple selections.
+ * Cursor animation in all the views.
+ * Can load more than one from from command line
+ * Track info, Toc info, Append/Insert Silence, Append/Insert Track,
+ *   they all are built for every child when needed.
+ * ...
+ *
  * Revision 1.7  2000/04/23 09:07:08  andreasm
  * * Fixed most problems marked with '//llanero'.
  * * Added audio CD edit menus to MDIWindow.
@@ -64,7 +82,7 @@
  *
  */
 
-static char rcsid[] = "$Id: SampleDisplay.cc,v 1.7 2000/04/23 09:07:08 andreasm Exp $";
+static char rcsid[] = "$Id: SampleDisplay.cc,v 1.10 2001/04/29 12:10:27 andreasm Exp $";
 
 #include <stdio.h>
 #include <limits.h>
@@ -418,6 +436,8 @@ void SampleDisplay::getColor(const char *colorName, Gdk_Color *color)
 
 void SampleDisplay::scrollTo()
 {
+  unsigned long minSample, maxSample;
+
   if (tocEdit_ == NULL)
     return;
 
@@ -427,19 +447,18 @@ void SampleDisplay::scrollTo()
   if (adjust->page_size == 0.0)
     return;
 
-  minSample_ = (unsigned long)adjust->value;
-  maxSample_ = (unsigned long)(adjust->value + adjust->page_size) - 1;
+  minSample = (unsigned long)adjust->value;
+  maxSample = (unsigned long)(adjust->value + adjust->page_size) - 1;
 
-  if (maxSample_ >= toc->length().samples()) {
-    maxSample_ = toc->length().samples() - 1;
-    if (maxSample_ <= (unsigned long)(adjust->page_size - 1))
-      minSample_ = 0;
+  if (maxSample >= toc->length().samples()) {
+    maxSample = toc->length().samples() - 1;
+    if (maxSample <= (unsigned long)(adjust->page_size - 1))
+      minSample = 0;
     else 
-      minSample_ = maxSample_ - (unsigned long)(adjust->page_size - 1);
+      minSample = maxSample - (unsigned long)(adjust->page_size - 1);
   }
 
-  updateSamples();
-  redraw(0, 0, width_, height_, 0);
+  viewModified(minSample, maxSample);
 }
 
 unsigned long SampleDisplay::pixel2sample(gint x)
@@ -1019,7 +1038,6 @@ void SampleDisplay::updateSamples()
       gint pos1;
       gint lastPosLeft, lastPosRight;
 
-
       //printf("Drawing exact, res=%ld pres=%g %ld-%ld\n", res, pres, minSample_, maxSample_);
 
       if (reader.seekSample(minSample_) == 0 &&
@@ -1060,8 +1078,12 @@ void SampleDisplay::updateSamples()
 	  if (pos != 0 || pos1 != 0)
 	    pixmap_.draw_line(drawGc_, long(di - pres), rcenter_ - (gint)pos,
 			       long(di), rcenter_ - pos1);
+
 	}
 
+	if (&pixmap_ == 0)
+	  cout << "null !!" << endl;
+
 	if (0 && (gint)di < sampleEndX_) {
 	  pos = sampleBuf[len -1].left() * halfHeight;
 	  pos /= SHRT_MAX;
@@ -1076,7 +1098,6 @@ void SampleDisplay::updateSamples()
 			       sampleEndX_, rcenter_ - (gint)pos);
 	}
       }
-
       delete[] sampleBuf;
     }
 
