@@ -16,47 +16,22 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: TocInfoDialog.cc,v $
- * Revision 1.4  2000/04/23 09:07:08  andreasm
- * * Fixed most problems marked with '//llanero'.
- * * Added audio CD edit menus to MDIWindow.
- * * Moved central storage of TocEdit object to MDIWindow.
- * * AudioCdChild is now handled like an ordinary non modal dialog, i.e.
- *   it has a normal 'update' member function now.
- * * Added CdTextTable modal dialog.
- * * Old functionality of xcdrdao is now available again.
- *
- * Revision 1.3  2000/04/16 20:31:20  andreasm
- * Added missing stdio.h includes.
- *
- * Revision 1.2  2000/02/20 23:34:54  llanero
- * fixed scsilib directory (files mising ?-()
- * ported xdao to 1.1.8 / gnome (MDI) app
- *
- * Revision 1.1.1.1  2000/02/05 01:40:15  llanero
- * Uploaded cdrdao 1.1.3 with pre10 patch applied.
- *
- * Revision 1.1  1999/08/21 14:17:39  mueller
- * Initial revision
- *
- */
-
-static char rcsid[] = "$Id: TocInfoDialog.cc,v 1.4 2000/04/23 09:07:08 andreasm Exp $";
 
 #include "TocInfoDialog.h"
 
+#include <gnome--.h>
+
 #include <stdio.h>
 #include <stddef.h>
 #include <string.h>
 #include <ctype.h>
 
 #include "TocEdit.h"
+//#include "TocEditView.h"
 #include "guiUpdate.h"
 #include "Toc.h"
 #include "CdTextItem.h"
 #include "TextEdit.h"
-#include "CdTextTable.h"
 
 #define MAX_CD_TEXT_LANGUAGE_CODES 22
 
@@ -301,28 +276,28 @@ TocInfoDialog::TocInfoDialog()
 
   Gtk::HButtonBox *bbox = new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD);
 
-  applyButton_ = new Gtk::Button(string(" Apply "));
+  applyButton_ = new Gnome::StockButton(GNOME_STOCK_BUTTON_APPLY);
   bbox->pack_start(*applyButton_);
   applyButton_->show();
   applyButton_->clicked.connect(SigC::slot(this,&TocInfoDialog::applyAction));
 
-  button = new Gtk::Button(string(" Cancel "));
+  button = new Gnome::StockButton(GNOME_STOCK_BUTTON_CLOSE);
   bbox->pack_start(*button);
   button->show();
-  button->clicked.connect(SigC::slot(this,&TocInfoDialog::cancelAction));
+  button->clicked.connect(SigC::slot(this,&TocInfoDialog::closeAction));
 
   get_action_area()->pack_start(*bbox);
   bbox->show();
   get_action_area()->show();
 
-  set_title(string("Disk Info"));
+  set_title(string("Project Info"));
 }
 
 TocInfoDialog::~TocInfoDialog()
 {
 }
 
-void TocInfoDialog::start(TocEdit *tocEdit)
+void TocInfoDialog::start(TocEdit *view)
 {
   if (active_) {
     get_window().raise();
@@ -331,7 +306,7 @@ void TocInfoDialog::start(TocEdit *tocEdit)
 
   active_ = 1;
 
-  update(UPD_ALL, tocEdit);
+  update(UPD_ALL, view);
   show();
 }
 
@@ -455,11 +430,9 @@ Gtk::VBox *TocInfoDialog::createCdTextPage(int n)
 {
   char buf[20];
   Gtk::Table *table = new Gtk::Table(11, 2, FALSE);
-  Gtk::VBox *vbox = new Gtk::VBox;
   Gtk::VBox *vbox1;
   Gtk::HBox *hbox;
   Gtk::Label *label;
-  Gtk::Button *button;
 
   sprintf(buf, "%d", n);
   cdTextPages_[n].label = new Gtk::Label(string(buf));
@@ -581,21 +554,9 @@ Gtk::VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].genreInfo), 1, 2, 10, 11);
   cdTextPages_[n].genreInfo->show();
 
-  vbox->pack_start(*table, FALSE);
-
-  button = new Gtk::Button(" Entry by Table ");
-  Gtk::HButtonBox *bbox = new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD);
-  bbox->pack_start(*button);
-  button->show();
-  button->clicked.connect(bind(slot(this, &TocInfoDialog::cdTextTableAction),
-			       n));
-  vbox->pack_start(*bbox, TRUE);
-  bbox->show();
-
-
   hbox = new Gtk::HBox;
-  hbox->pack_start(*vbox, TRUE, TRUE, 3);
-  vbox->show();
+  hbox->pack_start(*table, TRUE, TRUE, 3);
+  table->show();
 
   vbox1 = new Gtk::VBox;
   vbox1->pack_start(*hbox, TRUE, TRUE, 3);
@@ -606,24 +567,13 @@ Gtk::VBox *TocInfoDialog::createCdTextPage(int n)
   return vbox1;
 }
 
-void TocInfoDialog::cdTextTableAction(int language)
-{
-  if (tocEdit_ != NULL && tocEdit_->editable()) {
-    CdTextTable table(tocEdit_, language);
-
-    if (table.run()) {
-      guiUpdate();
-    }
-  }
-}
-
 gint TocInfoDialog::delete_event_impl(GdkEventAny*)
 {
   stop();
   return 1;
 }
 
-void TocInfoDialog::cancelAction()
+void TocInfoDialog::closeAction()
 {
   stop();
 }
@@ -643,27 +593,34 @@ void TocInfoDialog::clear()
   clearCdText();
 }
 
-void TocInfoDialog::update(unsigned long level, TocEdit *tocEdit)
+void TocInfoDialog::update(unsigned long level, TocEdit *view)
 {
   const Toc *toc;
 
   if (!active_)
     return;
 
-  tocEdit_ = tocEdit;
+  tocEdit_ = view;
 
-  if (tocEdit == NULL) {
+  if (view == NULL) {
     clear();
     return;
   }
 
+  string s(view->filename());
+  s += " - ";
+  s += APP_NAME;
+  if (view->tocDirty())
+    s += "(*)";
+  set_title(s);
+
   if (level & UPD_TOC_DATA) {
-    toc = tocEdit->toc();
+    toc = tocEdit_->toc();
     importData(toc);
   }
 
   if (level & UPD_EDITABLE_STATE) {
-    applyButton_->set_sensitive(tocEdit->editable() ? TRUE : FALSE);
+    applyButton_->set_sensitive(tocEdit_->editable() ? TRUE : FALSE);
   }
 }
 
