@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2000  Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,33 +16,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: TrackInfoDialog.cc,v $
- * Revision 1.3  2000/04/16 20:31:20  andreasm
- * Added missing stdio.h includes.
- *
- * Revision 1.2  2000/02/20 23:34:54  llanero
- * fixed scsilib directory (files mising ?-()
- * ported xdao to 1.1.8 / gnome (MDI) app
- *
- * Revision 1.1.1.1  2000/02/05 01:40:28  llanero
- * Uploaded cdrdao 1.1.3 with pre10 patch applied.
- *
- * Revision 1.1  1999/08/19 20:27:16  mueller
- * Initial revision
- *
- */
-
-static char rcsid[] = "$Id: TrackInfoDialog.cc,v 1.3 2000/04/16 20:31:20 andreasm Exp $";
 
 #include "TrackInfoDialog.h"
 
+#include <gnome--.h>
+
 #include <stdio.h>
 #include <stddef.h>
 #include <string.h>
 #include <ctype.h>
 
 #include "TocEdit.h"
+#include "TocEditView.h"
 #include "guiUpdate.h"
 #include "Toc.h"
 #include "Track.h"
@@ -61,7 +46,7 @@ TrackInfoDialog::TrackInfoDialog()
   Gtk::VBox *contents = new Gtk::VBox;
   Gtk::HBox *topBox = new Gtk::HBox;
 
-  tocEdit_ = NULL;
+  tocEditView_ = NULL;
   active_ = 0;
   trackNr_ = 0;
 
@@ -296,15 +281,15 @@ TrackInfoDialog::TrackInfoDialog()
 
   Gtk::HButtonBox *bbox = new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD);
 
-  applyButton_ = new Gtk::Button(string(" Apply "));
+  applyButton_ = new Gnome::StockButton(GNOME_STOCK_BUTTON_APPLY);
   bbox->pack_start(*applyButton_);
   applyButton_->show();
   applyButton_->clicked.connect(slot(this, &TrackInfoDialog::applyAction));
 
-  button = new Gtk::Button(string(" Cancel "));
+  button = new Gnome::StockButton(GNOME_STOCK_BUTTON_CLOSE);
   bbox->pack_start(*button);
   button->show();
-  button->clicked.connect(slot(this, &TrackInfoDialog::cancelAction));
+  button->clicked.connect(slot(this, &TrackInfoDialog::closeAction));
 
   get_action_area()->pack_start(*bbox);
   bbox->show();
@@ -317,7 +302,7 @@ TrackInfoDialog::~TrackInfoDialog()
 {
 }
 
-void TrackInfoDialog::start(TocEdit *tocEdit)
+void TrackInfoDialog::start(TocEditView *view)
 {
   if (active_) {
     get_window().raise();
@@ -326,7 +311,7 @@ void TrackInfoDialog::start(TocEdit *tocEdit)
 
   active_ = 1;
 
-  update(UPD_ALL, tocEdit);
+  update(UPD_ALL, view);
   show();
 }
 
@@ -440,7 +425,7 @@ gint TrackInfoDialog::delete_event_impl(GdkEventAny*)
   return 1;
 }
 
-void TrackInfoDialog::cancelAction()
+void TrackInfoDialog::closeAction()
 {
   stop();
 }
@@ -472,29 +457,36 @@ void TrackInfoDialog::clear()
   clearCdText();
 }
 
-void TrackInfoDialog::update(unsigned long level, TocEdit *tocEdit)
+void TrackInfoDialog::update(unsigned long level, TocEditView *view)
 {
   const Toc *toc;
 
   if (!active_)
     return;
 
-  tocEdit_ = tocEdit;
+  tocEditView_ = view;
 
-  if (tocEdit == NULL || !tocEdit->trackSelection(&selectedTrack_)) {
+  if (view == NULL || !view->trackSelection(&selectedTrack_)) {
     selectedTrack_ = 0;
     applyButton_->set_sensitive(FALSE);
     clear();
     return;
   }
 
+  string s(view->tocEdit()->filename());
+  s += " - ";
+  s += APP_NAME;
+  if (view->tocEdit()->tocDirty())
+    s += "(*)";
+  set_title(s);
+
   if (level & (UPD_TRACK_DATA | UPD_TRACK_MARK_SEL)) {
-    toc = tocEdit->toc();
+    toc = view->tocEdit()->toc();
     importData(toc, selectedTrack_);
-    applyButton_->set_sensitive(tocEdit->editable() ? TRUE : FALSE);
+    applyButton_->set_sensitive(view->tocEdit()->editable() ? TRUE : FALSE);
   }
   else if (level & UPD_EDITABLE_STATE) {
-    applyButton_->set_sensitive(tocEdit->editable() ? TRUE : FALSE);
+    applyButton_->set_sensitive(view->tocEdit()->editable() ? TRUE : FALSE);
   }
 }
 
@@ -530,10 +522,11 @@ void TrackInfoDialog::clearCdText()
 
 void TrackInfoDialog::applyAction()
 {
-  if (tocEdit_ == NULL || !tocEdit_->editable() || selectedTrack_ == 0)
+  if (tocEditView_ == NULL || !tocEditView_->tocEdit()->editable() ||
+      selectedTrack_ == 0)
     return;
 
-  exportData(tocEdit_, selectedTrack_);
+  exportData(tocEditView_->tocEdit(), selectedTrack_);
 
   guiUpdate(UPD_TRACK_DATA);
 }
