@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998, 1999  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2000  Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,19 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Log: TrackInfoDialog.h,v $
- * Revision 1.2  2000/02/20 23:34:54  llanero
- * fixed scsilib directory (files mising ?-()
- * ported xdao to 1.1.8 / gnome (MDI) app
- *
- * Revision 1.1.1.1  2000/02/05 01:38:55  llanero
- * Uploaded cdrdao 1.1.3 with pre10 patch applied.
- *
- * Revision 1.1  1999/08/19 20:27:16  mueller
- * Initial revision
- *
- */
 
 #ifndef __TRACK_INFO_DIALOG_H__
 #define __TRACK_INFO_DIALOG_H__
@@ -38,6 +25,7 @@
 
 class Toc;
 class TocEdit;
+class TocEditView;
 class TextEdit;
 
 class TrackInfoDialog : public Gtk::Dialog {
@@ -47,13 +35,13 @@ public:
 
   gint delete_event_impl(GdkEventAny*);
 
-  void update(unsigned long, TocEdit *);
+  void update(unsigned long, TocEditView *);
 
-  void start(TocEdit *);
+  void start(TocEditView *);
   void stop();
 
 private:
-  TocEdit *tocEdit_;
+  TocEditView *tocEditView_;
   int active_;
 
   int selectedTrack_;
@@ -91,7 +79,7 @@ private:
 
   CdTextPage cdTextPages_[8];
   
-  void cancelAction();
+  void closeAction();
   void applyAction();
 
   Gtk::VBox *createCdTextPage(int);
